<?php
/**
 * bll short summary.
 *
 * bll description.
 * YellowCube PrestaShop Modul
 *
 * Entwicklung: 	Zangger Websolutions
 * Web:			www.ihrewebagentur.ch
 * Projekt:		www.yellowcube.ihrewebagentur.ch
 * @version:		4.0
 */

include_once(dirname(__FILE__).'/dal/prestashopgateway.php');
include_once(dirname(__FILE__).'/dal/yellowcubegateway.php');
include_once(dirname(__FILE__).'/const.php');
include_once(dirname(__FILE__).'/common.php');
include_once(dirname(__FILE__).'/model/bar.php');
include_once(dirname(__FILE__).'/model/art.php');
include_once(dirname(__FILE__).'/model/productitem.php');
include_once(dirname(__FILE__).'/dal/mailprovider.php');
include_once(dirname(__FILE__).'/enum/artchangeflag.php');
include_once(dirname(__FILE__).'/enum/yellowcubestate.php');

class BLL
{

    public $prestaGateway;
    public $yellowGateway;

    public function __construct()
    {
        $this->prestaGateway = new PrestaShopGateway();
        $this->yellowGateway = new YellowCubeGateway();
    }


    /** Reads the BAR from YellowCube and compares it with the PrestaShop Inventory    
	*  Update: Bar Mails deaktivieren 
	*/
    public function processingBAR()
    {
        try {
            $barItems = array();

            $barItems = $this->processingArticles();

            $this->saveBar($barItems);

            $this->insertSyncsBar($barItems);

/**            $this->processingBarEmail($barItems); */

            PrestaShopGateway::updateConfig(Constants::LASTBARSYNC, date(Constants::TIMESTAMPFORMAT));
        } catch (Exception $e) {
            $message = htmlentities($e->getMessage());
            Logger::addLog("CronJob: Error Processing BAR from YellowCube. Message: ".$message, 4);
        }
    }


    /** Read and compares the Articles from PrestaShop and YellowCube */
    private function processingArticles()
    {
        $barObjectArray = array();

        $pendingStock = $this->getOpenWAB();

        $yellowCubeBar = $this->getBAR();
        $prestaStock =  $this->getPrestaStock();

        foreach ($prestaStock as $pro) {
            $bar = new Bar();

            $bar->presta_reference = $pro->reference;
            $bar->presta_stock = $pro->stock;

            if (array_key_exists($pro->reference, $pendingStock)) {
                $bar->pending_wab = $pendingStock[$pro->reference];
            }

            if (array_key_exists($pro->reference, $yellowCubeBar)) {
                $bar->yellowcube_stock = (int)$yellowCubeBar[$pro->reference]->QuantityUOM->_;
                $bar->description = $yellowCubeBar[$pro->reference]->ArticleDescription;
            }
            $bar->difference = ((int)$bar->presta_stock - (int)$bar->yellowcube_stock) + (int)$bar->pending_wab;
            array_push($barObjectArray, $bar);
        }
        return $barObjectArray;
    }

    /** Saves BAR Object into Database */
    private function saveBar($barItems)
    {
        foreach ($barItems as $value) {
            $this->prestaGateway->insertBar($value);
        }
    }


    /**
     * Reads the Prestashop article stock from database
     * @return ProductItem[] The PrestaShop Product Quantity
     */
    private function getPrestaStock()
    {
        $productArray = array();

        $products = $this->prestaGateway->getProducts();

        foreach ($products as $productar) {
            $product = (object)$productar;

            $productitem = new ProductItem();
            $productitem->id_product = $product->id_product;
            $productitem->reference = $product->reference;
            $productitem->id_product_attribute = '0';
            $productitem->stock = $this->prestaGateway->getProductStock(
                $productitem->id_product,
                $productitem->id_product_attribute
            );
            $productArray[$productitem->reference] = $productitem;

            $productAttribute = $this->prestaGateway->getProductAttributes($productitem->id_product);

            foreach ($productAttribute as $value) {
                $attributProduct = (object)$value;

                if ($attributProduct->reference != '') {
                    $productItemAttr = new ProductItem();
                    $productItemAttr->id_product = $product->id_product;
                    $productItemAttr->reference = $attributProduct->reference;
                    $productItemAttr->id_product_attribute = $attributProduct->id_product_attribute;
                    $productItemAttr->stock = $this->prestaGateway->getProductStock(
                        $productItemAttr->id_product,
                        $productItemAttr->id_product_attribute
                    );

                    $productArray[$productItemAttr->reference] = $productItemAttr;
                }
            }
        }

        return $productArray;

    }

    /**
    * Get the Productsquantity for Status 10 and 100 YellowCube Orders
    * @return array The OpenWAB Product Quantity
    */
    private function getOpenWAB()
    {
        $openWAB = $this->prestaGateway->getPendingOrders();

        $pendingStock = array();

        foreach ($openWAB as $wab) {
            $orderproducts = $this->prestaGateway->getOrderQuantity($wab['presta_reference']);

            foreach ($orderproducts as $product) {
                $quantity = (int)$product['product_quantity'];
                $refunded = (int)$product['product_quantity_refunded'];

                $count = $quantity - $refunded;

                if (array_key_exists($product['product_reference'], $pendingStock)) {
                    $pendingStock[$product['product_reference']] += + $count;
                } else {
                    $pendingStock[$product['product_reference']] = $count;
                }

            }
        }
        return $pendingStock;
    }

    /**
    * Gets the YellowCube Stock
    * @throws Exception BAR Exception
    * @return array Product Array
    * Update 05/2024: New foreach and Array
    */

private function getBAR(): array // Explicitly declare return type (array)
{
    $array = []; // Use empty array instead of array()

    $yellowCubeBar = $this->yellowGateway->getBAR();

    if ($yellowCubeBar->ArticleList ?? null) {
        $yellowCubeArticleList = $yellowCubeBar->ArticleList->Article;

        if ($yellowCubeArticleList ?? null) {
            foreach ($yellowCubeArticleList as $value) {
                $array[$value->ArticleNo] = $value;
            }

            PrestaShopGateway::updateConfig(
                Constants::YCBARSYNC,
                Common::createDateFromTimeStamp($yellowCubeBar->ControlReference->Timestamp)
            );
        } else {
            throw new Exception("Error getting BAR. Result is NULL");
        }
    }

    return $array;
}

    /**
*     Sends an Email to the Administrator when a difference in Stock is detected
*     @param mixed $bars BAR Object Array
*	 Update: Deactivate BAR Mails
*
*     private function processingBarEmail($bars)
*     {
*        $mail = new MailProvider();
*
*        $id_lang = $this->prestaGateway->getLangIDforBar();
*
*        foreach ($bars as $value) {
*            if ($value->difference != 0) {
*                $mail->sendBarEmail($value->presta_reference, $id_lang);
*            }
*        }
*    }
*    Stop Mails */


    /**
    * Returns the Data for the YellowCUbe Article TAB
    * @return array BAR Object Array
    */
    public function getGUIData()
    {
        try {
            $guiArray = array();

            $bars = PrestaShopGateway::getBar();

            foreach ($bars as $value) {
                $bar = new Bar();

                $bar->description = $value['description'];
                $bar->difference = $value['difference'];
                $bar->pending_wab = $value['pending_wab'];
                $bar->presta_stock = $value['presta_stock'];
                $bar->presta_reference = $value['presta_reference'];
                $bar->yellowcube_stock =  $value['yellowcube_stock'];

                $bar->statuscode =  (int)$value['statuscode'];
                $bar->statustext =  $value['statustext'];

                $bar->isChecked = $this->checkIfArtIsAvaiable($bar->presta_reference);

                array_push($guiArray, $bar);
            }

            return $guiArray;
        } catch (Exception $e) {
            $message = htmlentities($e->getMessage());
            Logger::addLog("Failed to load Data for GUI. Message: ".$message, 4);
        }
    }

    /**
    * Summary of checkIfArtIsAvaiable
    * @param string $presta_reference The PrestaShop Order Reference
    * @return bool True: Product is updated ||| False: Product is not updated
    */
    private function checkIfArtIsAvaiable($presta_reference)
    {
        $bool = false;

        $data = $this->prestaGateway->getYellowCubeSyncForProduct($presta_reference);

        if ($data != null) {
            $syncDate = DateTime::createFromFormat("Y-m-d H:i:s", $data[0]['timpestamp']);

            if ($this->prestaGateway->checkIfProductAttribute($presta_reference)) {
                $prestaDate = $this->prestaGateway->getUpdateDateForProductVariant($presta_reference);

                if ($prestaDate > $syncDate) {
                    $bool = true;
                }
            } else {
                $prestaDate = $this->prestaGateway->getUpdateDateForProduct($presta_reference);

                if ($prestaDate > $syncDate) {
                    $bool = true;
                }
            }
        } else {
            $bool = true;
        }

        return $bool;
    }

    /**
    * Check if a new Product is added in PrestaShop
    */
    public function checkForNewProducts()
    {
        try {
			
			
            $barArray = array();
            $prestaArray = array();
            $newProductArray = array();

            $products = $this->getPrestaStock();
            $bars = $this->getGUIData();

            foreach ($products as $product) {
                if ($product->reference != "") {
                    $prestaArray[] = $product->reference;
                }
            }

            foreach ($bars as $bar) {
                $barArray[] = $bar->presta_reference;
            }

            foreach ($prestaArray as $prestaRef) {
                if (!in_array($prestaRef, $barArray)) {
                    $newProductArray[] = $prestaRef;
                }
            }

			Logger::addLog("Start Add new Products", 1, null, null, null, true);
            foreach ($newProductArray as $newProduct) {
				
				Logger::addLog("Adding new Product: ".$newProduct, 1, null, null, null, true);
				
                $newBar = new Bar();

                $newBar->presta_reference = $newProduct;
                $newBar->presta_stock = $products[$newProduct]->stock;

                $this->prestaGateway->insertBar($newBar);
            }
			
			Logger::addLog("Finished adding new Products", 1, null, null, null, true);
			
        } catch (Exception $e) {
            $message = htmlentities($e->getMessage());
            Logger::addLog("Failed to check for new Products. Message: ".$message, 4, null, null, null, true);
        }
    }

    // ART

    /**
     * Sends ARTs to YellowCube
     * @param string $product_references An array of PrestaShop product references
     */
    public function syncArt($product_reference)
    {
        try {
            $art = $this->fillArtData($product_reference);
            if (array_key_exists('de', $art->articleDescription)) {
                $this->yellowGateway->sendART($art);
                $this->insertYellowCubeSync($product_reference);
            } else {
                $this->prestaGateway->updateBARWithArtAnswer(
                    9999,
                    YellowCubeState::ERROR,
                    'Please update the product description for language de',
                    $art->articleNo,
                    ""
                );
            }

            return true;
        } catch (Exception $e) {
            $message = htmlentities($e->getMessage());
            Logger::addLog("Error Processing ART for YellowCube. Message: ".$message, 4);

            return false;
        }
    }

    /**
    * Sends ARTs to YellowCube
    * @param array $product_references An array of PrestaShop product references
    */
    public function processingART($product_references)
    {
        try {
            $artArray = array();
            $artArray = $this->fillArtDatas($product_references);
            foreach ($artArray as $art) {
                $this->yellowGateway->sendART($art);
            }
            $this->insertSyncsArt($artArray);
        } catch (Exception $e) {
            $message = htmlentities($e->getMessage());
            Logger::addLog("Error Processing ART for YellowCube. Message: ".$message, 4);
        }
    }

    /**
    * Deactivates ARTs in YellowCube
    * @param array $product_references An array of PrestaShop product references
    */
    public function deactivateART($product_reference)
    {
        try {
            $art = $this->fillArtData($product_reference);

            $art->changeFlag = ARTChangeFlag::DELETE;
            $this->yellowGateway->sendART($art);
            return true;
        } catch (Exception $e) {
            $message = htmlentities($e->getMessage());
            Logger::addLog("Error Processing ART for YellowCube. Message: ".$message, 4);
            return false;
        }
    }

    /**
     * Gets the Productdata from PrestaShop
     * @param array $product_references An array of PrestaShop product references
     * @return array an Array with Product Data
     */
    private function fillArtDatas($product_references)
    {
        $artArray = array();

        foreach ($product_references as $reference) {
            array_push($artArray, $this->fillArtData($reference));
        }

        return $artArray;
    }

    /**
     * Gets the Productdata from PrestaShop
     * @param string $product_references An array of PrestaShop product references
     * @return object an Array with Product Data
     */
    private function fillArtData($product_references)
    {
        $art = null;
        if ($this->prestaGateway->checkIfProductAttribute($product_references)) {
            $art = $this->createArtObjectForProductVariant($product_references);
        } else {
            $art = $this->createArtObjectforProduct($product_references);
        }

        return $art;
    }


    /**
     * Fills the propertys of the art object for a product
     * @param string $reference PresteShop order reference
     * @return Art The Art object for YellowCube
     */
    private function createArtObjectForProduct($reference)
    {
        $product_id = (int)$this->prestaGateway->getProductIdPerReference($reference);
        $product = new Product($product_id);
        $langs = $this->prestaGateway->getLangArrayForArt();

        $art = $this->createStandartArt($reference);

        $art->ean = $product->ean13;
        $art->length = number_format(doubleval($product->depth), 3, '.', "");
        $art->width = number_format(doubleval($product->width), 3, '.', "");
        $art->height = number_format(doubleval($product->height), 3, '.', "");

        $art->volume = number_format((float)$art->length * $art->width * $art->height, 3, '.', "");

        $art->grossWeight = number_format(doubleval($product->weight), 3, '.', "");

        $art->netWeight = number_format(doubleval($product->weight), 3, '.', "");
        $art->articleDescription = $this->createDescriptionArrayForProduct($langs, $product->name);
        $art->changeFlag = $this->getChangeFlag($reference);

        return $art;
    }

    /**
     * Fills the propertys of the art object for a product variant
     * @param string $reference PresteShop order reference
     * @return Art The Art object for YellowCube
     */
    private function createArtObjectForProductVariant($reference)
    {
        $data = $this->prestaGateway->getProductVariantData($reference);

        $product = new Product($data[0]['id_product']);
        $attributes = $this->prestaGateway->getProductAttribute($data[0]['id_product_attribute']);
        $langs = $this->prestaGateway->getLangArrayForArt();

        $art = $this->createStandartArt($reference);

        $art->ean = $data[0]['ean13'];
        $art->length = number_format(doubleval($product->depth), 3, '.', "");
        $art->width = number_format(doubleval($product->width), 3, '.', "");
        $art->height = number_format(doubleval($product->height), 3, '.', "");

        $art->volume = number_format((float)($art->length * $art->width * $art->height), 3, '.', "");

        $art->grossWeight = number_format(doubleval($product->weight), 3);

        $art->netWeight = number_format(doubleval($product->weight), 3);
        $art->articleDescription = $this->createDescriptionArrayForProductVariant($langs, $product->name, $attributes);
        $art->changeFlag = $this->getChangeFlag($reference);

        return $art;
    }

    /**
     * Fills the propertys of the standart art object
     * @param string $reference PresteShop order reference
     * @return Art The Art object for YellowCube
     */
    private function createStandartArt($reference)
    {
        $art = new Art();

        $art->depositorNo = PrestaShopGateway::getConfig(Constants::DEPOSITOR_NO);
        $art->plantID = PrestaShopGateway::getConfig(Constants::PLANT);
        $art->articleNo = $reference;
        $art->alternateUnitISO = PrestaShopGateway::getConfig(Constants::QUANTITY_ISO);
        $art->altNumeratorUOM = Constants::ALTNUMERATORUOM;
        $art->altDenominatorUOM = Constants::ALTDENOMINATORUOM;

        return $art;
    }

    /**
     * Create an description array for product variant
     * @param array $langs An array with installed languages
     * @param array $product_desc An Array with products descriptions
     * @param array $attribut_desc An Array with attribute descriptions
     * @return string[] the description array
     */
    private function createDescriptionArrayForProductVariant($langs, $product_desc, $attribut_desc)
    {
        $finalArray = array();

        foreach ($langs as $lang) {
            $att_Desc = "";
            foreach ($attribut_desc as $value) {
                $att = new Attribute((int)$value['id_attribute']);
                $att_Desc = $att_Desc." ".$att->name[(int)$lang['id_lang']];
            }
            $finalDesc = strip_tags($product_desc[(int)$lang['id_lang']]).$att_Desc;
            $finalArray[$lang['iso_code']] = Tools::substr($finalDesc, 0, 40);
        }
        return $finalArray;
    }

    /**
     * Create an description array for product
     * @param array $langs An array with installed languages
     * @param array $product_desc An Array with products descriptions
     * @return string[] the description array
     */
    private function createDescriptionArrayForProduct($langs, $product_desc)
    {
        $finalArray = array();
        foreach ($langs as $lang) {
            $finalArray[$lang['iso_code']] = Tools::substr(strip_tags($product_desc[(int)$lang['id_lang']]), 0, 40);
        }
        return $finalArray;
    }

    /**
     * Get the ChangeFlag for YellowCube
     * @param mixed $presta_reference PresteShop order reference
     * @return string ChangeFlag Insert or Update
     */
    private function getChangeFlag($presta_reference)
    {
        $data = $this->prestaGateway->getYellowCubeSyncForProduct($presta_reference);

        if ($data != null) {
            return ARTChangeFlag::UPDATE;
        } else {
            return ARTChangeFlag::INSERT;
        }
    }

    /**
     * Insert bar syncs references
     * @param array $bars Bar object array
     */
    private function insertSyncsBar($bars)
    {
        $references = array();
        foreach ($bars as $bar) {
            if ($bar->description != '') {
                $references[] = $bar->presta_reference;
            }
        }

        $this->insertYellowCubeSyncs($references);
    }

    /**
     * Insert art syncs references
     * @param array $arts ART object array
     */
    private function insertSyncsArt($arts)
    {
        $references = array();
        foreach ($arts as $art) {
            $references[] = $art->articleNo;
        }

        $this->insertYellowCubeSyncs($references);
    }

    /**
     * Summary of insertYellowCubeSyncs
     * @param mixed $presta_references PresteShop order reference
     */
    private function insertYellowCubeSyncs($presta_references)
    {
        foreach ($presta_references as $reference) {
            $this->insertYellowCubeSync($reference);
        }
    }

    /**
     * Summary of insertYellowCubeSyncs
     * @param string $presta_reference PresteShop order reference
     */
    private function insertYellowCubeSync($presta_reference)
    {
        $data = $this->prestaGateway->getYellowCubeSyncForProduct($presta_reference);
        if ($data != null) {
            $this->prestaGateway->updateYellowCubeSync($presta_reference);
        } else {
            $this->prestaGateway->insertYellowCubeSync($presta_reference);
        }
    }


    /**
     * Updates a statuscode 10 ART
     */
    public function updatePendingART()
    {
        try {
            $pendingArts = $this->prestaGateway->getPendingART();

            foreach ($pendingArts as $pendingArt) {
                $this->yellowGateway->updateART($pendingArt['presta_reference'], $pendingArt['yc_reference']);
            }
        } catch (Exception $e) {
            $message = htmlentities($e->getMessage());
            Logger::addLog("Error: $e / Error updating pending ART from YellowCube. Message: ".$message, 4);
        }
    }
    /**
     * Removes unused ProductReferences from BAR table
     */
    public function removeUnusedProductsFromBar()
    {
        try {
            $bars = PrestaShopGateway::getBar();
            foreach ($bars as $value) {
                if (!PrestaShopGateway::checkIfPrestaReferenceStillAvailable($value['presta_reference'])) {
                    PrestaShopGateway::removeProductFromBar($value['presta_reference']);
                }
            }
        } catch (Exception $e) {
            $message = htmlentities($e->getMessage());
            Logger::addLog("Not able to delete UnusedProducts: ".$message, 4);
        }
    }
}
