<?php
/**
 * PrestaShopGateway short summary.
 *
 * PrestaShopGateway description.
 *
 * Entwicklung: 	Zangger Websolutions
 * Web:			www.ihrewebagentur.ch
 * Projekt:		www.yellowcube.ihrewebagentur.ch
 * @version:		4.0
 */

include_once(dirname(__FILE__).'/../orderposition.php');
include_once(dirname(__FILE__).'/../exceptions.php');


class PrestaShopGateway
{

    public function __construct()
    {
    }

    /**
    * Load a config value from yellowcube_config table
    * @param string $name Config Name
    * @return mixed the config value
    */
    public static function getConfig($name)
    {
        $value = Db::getInstance()->getValue('
                SELECT `value`
                FROM `'._DB_PREFIX_.'yellowcube_configuration`
                WHERE name = "'.pSQL($name).'"
            ');
        return $value ? $value : false;
    }

    /**
    * Summary of insertConfig
    * @param mixed $name Config Name
    * @param mixed $value the config value
    */
    public static function insertConfig($name, $value)
    {
        $config_exist = Db::getInstance()->getValue('
            SELECT COUNT(*)
            FROM `'._DB_PREFIX_.'yellowcube_configuration`
            WHERE `name` = "'.pSQL($name).'"
        ');
        if (!$config_exist) {
            Db::getInstance()->Execute('
                INSERT INTO `'._DB_PREFIX_.'yellowcube_configuration`
                (`name`, `value`)
                VALUES ("'.pSQL($name).'", "'.pSQL($value).'")
            ');
        }
    }

    /**
    * Update exist config in yellowcube_config table.
    * @param mixed $name Config Name
    * @param mixed $value new value
    */
    public static function updateConfig($name, $value)
    {
        $config_exist = Db::getInstance()->getValue('
            SELECT COUNT(*)
            FROM `'._DB_PREFIX_.'yellowcube_configuration`
            WHERE `name` = "'. pSQL($name) .'"
        ');
        if ($config_exist) {
            Db::getInstance()->Execute('
                UPDATE `'._DB_PREFIX_.'yellowcube_configuration`
                SET `value` = "' . pSQL($value) . '"
                WHERE `name` = "'  .pSQL($name). '"
            ');
        }
    }

    /**
    * Summary of getAdressArray
    * @param mixed $param the adress id
    * @return array adress data
    */
    public function getAdressArray($param)
    {
        $statement = "SELECT `address1`, `postcode`, `firstname`, `lastname`,
            `city`, `id_country`, `company`, `address2`, `other`, `dni` FROM "
            ._DB_PREFIX_."address WHERE `id_address` = ".(int)$param;
        $addressvalues = Db::getInstance()->ExecuteS($statement);

        $countryISO = Db::getInstance()->ExecuteS(
            'SELECT `iso_code` FROM '._DB_PREFIX_.'country WHERE `id_country` = '.(int)$addressvalues[0]['id_country']
        );

        $values = array(
            "name1"    => $addressvalues[0]['firstname'].' '.$addressvalues[0]['lastname'],
            "address1"    => $addressvalues[0]['address1'],
            "postcode"  => $addressvalues[0]['postcode'],
            "city"  => $addressvalues[0]['city'],
            "iso_code" => $countryISO[0]['iso_code'],
            "address2" => $addressvalues[0]['address2'],
            "company" => $addressvalues[0]['company'],
            "other" => $addressvalues[0]['other'],
            "dni" => $addressvalues[0]['dni'],
        );
        return $values;
    }

    /**
    * Select the Error Orders for the BackOffice page
    * @return array data from yellowcube_orders table
    */
    public static function displayOrders()
    {
        $orders = Db::getInstance()->ExecuteS(
            "SELECT * FROM "._DB_PREFIX_."yellowcube_orders WHERE `statuscode` > 100 ORDER BY `timestamp` DESC"
        );
        return $orders;
    }

    /**
    * Select the pending orders for the backoffice page
    * @return array The yellowcube_orders dataArray
    */
    public static function getPendingOrders()
    {
        $statement = "SELECT * FROM "._DB_PREFIX_."yellowcube_orders WHERE `statuscode` =
            10 OR (`statuscode` = 100 AND `tracking` IS NULL) ORDER BY `timestamp` DESC";
        $orders = Db::getInstance()->ExecuteS($statement);
        return $orders;
    }

    /**
    * Select the ISO code from lang table
    * @param string $param ID from lang table
    * @return mixed language ISO code
    */
    public function getLangISO($param)
    {
        $value = Db::getInstance()->ExecuteS(
            'SELECT `iso_code` FROM '._DB_PREFIX_.'lang WHERE `id_lang` = '.(int)$param
        );
        if (empty($value)) {
            return false;
        }
        return $value[0]['iso_code'];
    }

    /**
     * Summary of getCarrierName
     * @param string $param Carrier ID
     * @return string Carrier Name
     */
    public function getCarrierName($param)
    {
        $value = Db::getInstance()->ExecuteS(
            'SELECT `name` FROM '._DB_PREFIX_.'carrier WHERE `id_carrier` = '.(int)$param
        );

        if (empty($value)) {
            return false;
        }

        return $value[0]['name'];
    }

    /**
     * Get the orderpositions(Article) from an order
     * @param mixed $reference PrestaShop order reference
     * @return array An array with orderposition objects
     */
    public function getOrdersPosition($reference)
    {
        $finalOrderPositionArray = array();
        $order_id = Db::getInstance()->ExecuteS(
            "SELECT `id_order` FROM "._DB_PREFIX_."orders WHERE `reference` = '".(string)$reference."'"
        );
        if (empty($order_id)) {
            return false;
        }
        $orderPositionIdArray = Db::getInstance()->ExecuteS(
            'SELECT `product_reference`, `product_ean13`,`product_quantity`, `product_quantity_refunded` FROM '
            ._DB_PREFIX_.'order_detail WHERE `id_order` = '.(int)$order_id[0]['id_order']
        );
        $number = 10;
        foreach ($orderPositionIdArray as $position) {
            $orderPosition = new OrderPosition(
                PrestaShopGateway::getConfig(Constants::PLANT),
                PrestaShopGateway::getConfig(Constants::QUANTITY_ISO)
            );
            $orderPosition->Quantity = $position['product_quantity'] - $position['product_quantity_refunded'];
            $orderPosition->PosNo = $number;
            $orderPosition->Ean = $position['product_ean13'];
            $orderPosition->ArticleNo = $position['product_reference'];
            if ($orderPosition->Quantity > 0) {
                array_push($finalOrderPositionArray, $orderPosition);
            }
            $number = $number + 10;
        }
        return $finalOrderPositionArray;
    }

    /**
     * Insert a YellowCube order entry in the yellowcube_orders table
     * @param mixed $yc_reference Yellowcube reference
     * @param mixed $presta_reference The PrestaShop order reference
     * @param mixed $timestamp Yellowcube eventtimestamp
     * @param mixed $statuscode Yellowcube statuscode
     * @param mixed $stateID yellowcube_state ID
     * @param mixed $statustext Yellowcube text
     * @param mixed $tracking Yellowcube trackingnumber
     */
    public static function insertYellowCubeOrder(
        $yc_reference,
        $presta_reference,
        $timestamp,
        $statuscode,
        $stateID,
        $statustext,
        $tracking = ""
    ) {
        $statement = "INSERT INTO "._DB_PREFIX_."yellowcube_orders(`yc_reference`, `presta_reference`, `timestamp`,
            `statuscode`, `stateID`, `statustext`, `tracking`) VALUES ('"
            .(string)$yc_reference."','".(string)$presta_reference."','".(string)$timestamp."',"
            .(int)$statuscode.",".(int)$stateID.",'".(string)$statustext."','".(string)$tracking."')";
        Db::getInstance()->Execute($statement);
    }

    /**
     * Update an existing entry with new data from YellowCube
     * @param mixed $id yellowcube_order id
     * @param mixed $yc_reference Yellowcube reference
     * @param mixed $timestamp Yellowcube eventtimestamp
     * @param mixed $statuscode Yellowcube statuscode
     * @param mixed $stateID yellowcube_state ID
     * @param mixed $statustext Yellowcube text
     */
    public static function updateYellowCubeOrder($id, $yc_reference, $timestamp, $statuscode, $stateID, $statustext)
    {
        $statement = 'UPDATE '._DB_PREFIX_.'yellowcube_orders SET `yc_reference`="'
            .(string)$yc_reference.'",`timestamp`="'.(string)$timestamp.'",`statuscode`="'
            .(int)$statuscode.'",`stateID`="'.(int)$stateID.'",`statustext`="'
            .(string)$statustext.'" WHERE `id` = "'.(int)$id.'"';
        Db::getInstance()->Execute($statement);
    }

    /**
     * Update an existing entry with new data from YellowCube
     * @param mixed $id yellowcube_order id
     * @param mixed $statuscode Yellowcube statuscode
     * @param mixed $stateID yellowcube_state ID
     * @param mixed $statustext Yellowcube text
     */
    public static function setYellowCubeOrderToError($id, $statuscode, $stateID, $statustext)
    {
        Db::getInstance()->Execute('UPDATE '._DB_PREFIX_.'yellowcube_orders SET
            `statuscode`="'.(int)$statuscode.'",
            `stateID`="'.(int)$stateID.'",
            `statustext`="'.(string)$statustext.'"
            WHERE `id` = "'.(int)$id.'"');
    }

    /**
     * Update an existing entry with new data from YellowCube
     * @param mixed $id yellowcube_order id
     * @param mixed $statuscode Yellowcube statuscode
     * @param mixed $stateID yellowcube_state ID
     * @param mixed $statustext Yellowcube text
     */
    public static function setYellowCubeOrderToErrorByRef($ref, $statuscode, $stateID, $statustext)
    {
        $id = Db::getInstance()->ExecuteS(
            "SELECT id FROM "._DB_PREFIX_."yellowcube_orders WHERE `presta_reference` ='".(string)$ref."'"
        );

        PrestaShopGateway::setYellowCubeOrderToError($id, $statuscode, $stateID, $statustext);
    }

    /**
     * Upate the trackingnummer from WARData
     * @param mixed $reference Prestashop reference
     * @param mixed $postalShipmentNo PostalShipmentNo from YellowCube
     * @param mixed $text Statustext
     */
    public static function updateTrackingNumber($reference, $postalShipmentNo, $text)
    {
        Db::getInstance()->Execute(
            "UPDATE "._DB_PREFIX_."yellowcube_orders SET `tracking`='"
            .(string)$postalShipmentNo."', `stateID`="
            .(int)YellowCubeState::FINISHED.", `statustext`='"
            .(string)$text."' WHERE `presta_reference` = '"
            .(string)$reference."'"
        );
    }

    /**
     * Summary of updateOrderTrackingNumber
     * @param mixed $id Order id
     * @param mixed $postalShipmentNo PostalShipmentNo from YellowCube
     */
    public static function updateOrderTrackingNumber($id, $postalShipmentNo)
    {
        $statement = "UPDATE "._DB_PREFIX_."order_carrier SET `tracking_number`='"
            .(string)$postalShipmentNo."' WHERE `id_order` = ".(int)$id;
        Db::getInstance()->Execute($statement);
    }

    /**
     * Get the order from orders table by reference
     * @param mixed $orderReference PrestaShop Reference
     * @return mixed The order object
     */
    public static function getOrderByReference($orderReference)
    {
        $value = Db::getInstance()->ExecuteS(
            "SELECT * FROM "._DB_PREFIX_."orders WHERE `reference` = '".(string)$orderReference."'"
        );
        return $value;
    }

    /**
     * Upate the OrderState for Order
     * @param mixed $orderID Order ID
     * @param mixed $stateID State ID
     */
    public static function updateOrderStateByID($orderID, $stateID)
    {
        $objOrder = new Order($orderID);
        $history = new OrderHistory();
        $history->id_order = (int)$objOrder->id;
        $history->changeIdOrderState($stateID, (int)($objOrder->id));
        $history->add(true);
    }

    /**
     * Upate the OrderState for Order
     * @param mixed $ycID Order ID
     * @param mixed $stateID State ID
     */
    public static function updateOrderStateByYCID($ycID, $stateID)
    {
        $prest = new PrestaShopGateway();
        PrestaShopGateway::updateOrderStateByID($prest->getOrderIDByYellowCubeID($ycID), $stateID);
    }


    /**
    * Deletes an existing Entry from PrestaShop
    * @param mixed $id yellowcue_order table ID
    */
    public function deleteYellowCubeOrder($id)
    {
        Db::getInstance()->Execute("DELETE FROM "._DB_PREFIX_."yellowcube_orders WHERE `id` = ".(int)$id);
    }

    /**
    * Get the Orderreference from yellowcube_orders table with ID
    * @param string $id The record ID
    * @return string PrestaShop Order Reference
    */
    public function getOrderReferencefromYC($id)
    {
        $value = Db::getInstance()->ExecuteS(
            'SELECT `presta_reference`, `id` FROM '._DB_PREFIX_.'yellowcube_orders WHERE `id` = '.(int)$id
        );
        return $value[0]['presta_reference'];
    }

    /**
    * Get all non full processed YC orders from table
    * @return mixed an array with all non full processed YC orders
    */
    public function getNotFullProcessedYCOrders()
    {
        $value = Db::getInstance()->ExecuteS(
            'SELECT `yc_reference`, `id` FROM '._DB_PREFIX_.'yellowcube_orders WHERE `statuscode` < 100'
        );
        return $value;
    }

    /**
    * Get the order from yellowcube_orders table by reference
    * @param string $ref PrestaShop Reference
    * @return mixed The ycOrder object
    */
    public function getYCOrderbyReference($ref)
    {
        $value = Db::getInstance()->ExecuteS(
            "SELECT * FROM "._DB_PREFIX_."yellowcube_orders WHERE `presta_reference` ='".(string)$ref."'"
        );
        return $value;
    }

    /**
    * Get the order from orders table by id
    * @param mixed $orderId Order ID
    * @return mixed The order object
    */
    public function getOrderByID($orderId)
    {
        $value = Db::getInstance()->ExecuteS('SELECT * FROM '._DB_PREFIX_.'orders WHERE `id_order` = '.(int)$orderId);
        return $value;
    }

    /**
    * Get all WAR ready YC orders from table
    * @return mixed an array with all WAR ready YC orders
    */
    public function getWAROrders()
    {
        $statement = "SELECT `presta_reference` FROM "._DB_PREFIX_.
            "yellowcube_orders WHERE `stateID` = 3";
        $value = Db::getInstance()->ExecuteS($statement);
        return $value;
    }





    /**
    * Get the State ID for YellowCubeDelivered State
    */
    public static function setYellowCubeShippedID()
    {
        $statement = "SELECT `id_order_state` from "._DB_PREFIX_.
            "order_state_lang WHERE `name` = '".(string)Constants::YELLOWCUBESHIPPED."'";
        $value = Db::getInstance()->ExecuteS($statement);
        PrestaShopGateway::updateConfig(Constants::YELLOWCUBESHIPPEDID, $value[0]['id_order_state']);
    }


    /**
    * Get orders with specific state
    * @param mixed $statusID State ID
    * @return array array with order objects
    */
    public function getOrdersByStatus($statusID)
    {
        $orders = Db::getInstance()->ExecuteS(
            "SELECT * from "._DB_PREFIX_."orders WHERE `current_state` = ".(int)$statusID
        );
        return $orders;
    }


    /**
    * Get the order id with the yellowcube_orders row id
    * @param mixed $id the yellowcube_orders row id
    * @return string the order id
    */
    public function getOrderIDByYellowCubeID($id)
    {
        $orderRef = $this->getOrderReferencefromYC($id);
        return $this->getOrderIDByReference($orderRef);
    }

    /**
    * Get the order id by order reference
    * @param mixed $ref order reference
    * @return string the order id
    */
    public function getOrderIDByReference($ref)
    {
        $value = Db::getInstance()->ExecuteS(
            "SELECT `id_order` FROM "._DB_PREFIX_."orders WHERE `reference` = '".(string)$ref."'"
        );
        return $value[0]['id_order'];
    }

    /**
     * Generate the DeliveryPDF for YellowCube
     * @param string $id_order >The Order ID
     * @throws PrestaShopException When order object cannot be loaded
     * @return bytestream DeliveryPDF byteStream
     */
    public function generateDeliverySlipPDFByIdOrder($id_order)
    {
        $order = new Order((int)$id_order);
        if (!Validate::isLoadedObject($order)) {
            throw new PrestaShopException('Can\'t load Order object');
        }
        $this->checkIfDeliverySlipIsAvaiable($order);
        $order_invoice_collection = $order->getInvoicesCollection();
        return $this->generatePDF($order_invoice_collection, PDF::TEMPLATE_DELIVERY_SLIP);
    }

    /**
    * Check if a deliverySlip can be generated or not
    * @param mixed $order the order object
    * @throws NoDeliveryException When no deliveryslip can be generated
    */
    private function checkIfDeliverySlipIsAvaiable($order)
    {
        $orderState = new OrderState($order->getCurrentState());
        if ($orderState->delivery == 0) {
            throw new NoDeliveryException(
                'Can\'t generate delivery Slip with OrderState: '.$orderState->name[1],
                $order->reference
            );
        }
    }

    /**
    * Generate the DeliveryPDF
    * @param mixed $object order invoice collection
    * @param mixed $template PDF Template
    * @return mixed PDF bytestream
    */
    private function generatePDF($object, $template)
    {
        $pdf = new PDF($object, $template, Context::getContext()->smarty);
        $pdfByteStream = $pdf->render(false);
        //file_put_contents('C:\temp\test.pdf', $pdfByteStream);
        return $pdfByteStream;
    }

    /**
    * Get Order positions
    * @param mixed $reference the order reference
    * @return array the order positions
    */
    public function getOrderQuantity($reference)
    {
        $order_id = Db::getInstance()->ExecuteS(
            "SELECT `id_order` FROM "._DB_PREFIX_."orders WHERE `reference` = '".(string)$reference."'"
        );
        if (empty($order_id)) {
            return false;
        }

        $statement = 'SELECT `product_reference`, `product_quantity`, `product_quantity_refunded` FROM '._DB_PREFIX_.
            'order_detail WHERE `id_order` = '.(int)$order_id[0]['id_order'];

        $orderPositionArray = Db::getInstance()->ExecuteS($statement);
        return $orderPositionArray;
    }

    /**
     * select all products
     * @return array products
     */
    public function getProducts()
    {
        $products = Db::getInstance()->ExecuteS('SELECT `id_product`, `reference` FROM '._DB_PREFIX_.'product');
        return $products;
    }

    /**
     * Get product attributes
     * @param mixed $productID the product id
     * @return array product attributes
     */
    public function getProductAttributes($productID)
    {
        $productAttributes = Db::getInstance()->ExecuteS(
            "SELECT `id_product_attribute`, `reference` FROM "
            ._DB_PREFIX_."product_attribute where `id_product` = ".(int)$productID
        );
        return $productAttributes;
    }

    /**
     * Get the current product stock from PrestaShop
     * @param mixed $productID product id
     * @param mixed $productAttributeID attribute id
     * @return int quantity
     */
    public function getProductStock($productID, $productAttributeID)
    {
        $statement = "SELECT `quantity` FROM "._DB_PREFIX_."stock_available WHERE `id_product` = "
            .(int)$productID." AND `id_product_attribute` = ".(int)$productAttributeID;
        $stock = Db::getInstance()->ExecuteS($statement);
        return $stock[0]['quantity'];
    }

    /**
     * Insert new Bar in DB or update existing one
     * @param Bar $bar bar object
     */
    public function insertBar($bar)
    {
        $statement = "SELECT * FROM "._DB_PREFIX_."yellowcube_bar WHERE `presta_reference` = '"
            .(string)$bar->presta_reference."'";
        $value = Db::getInstance()->ExecuteS($statement);

        $statement = "";
        if (count($value) == 0) {
            $statement = "INSERT INTO "._DB_PREFIX_."yellowcube_bar(`presta_reference`, `description`, `presta_stock`,
                `yellowcube_stock`, `pending_wab`, `difference`, `stateID`) VALUES ('"
                .(string)$bar->presta_reference."','".(string)$bar->description."','".(string)$bar->presta_stock."','"
                .(string)$bar->yellowcube_stock."','".(string)$bar->pending_wab."','"
                .(string)$bar->difference."',".YellowCubeState::UNDEFINED.")";
            Db::getInstance()->Execute($statement);
        } else {
            $statement = "UPDATE "._DB_PREFIX_."yellowcube_bar SET
                `presta_reference`='".(string)$bar->presta_reference."',
                `description`='".(string)$bar->description."',
                `presta_stock`='".(string)$bar->presta_stock."',
                `yellowcube_stock`='".(string)$bar->yellowcube_stock."',
                `pending_wab`='".(string)$bar->pending_wab."',
                `difference`='".(string)$bar->difference."'
                 WHERE `presta_reference` = '".(string)$bar->presta_reference."'";
            Db::getInstance()->Execute($statement);
        }
    }

    /**
     * Get all bar
     * @return array bar array
     */
    public static function getBar()
    {
        return Db::getInstance()->ExecuteS("SELECT * FROM "._DB_PREFIX_."yellowcube_bar");
    }

    /**
     * Get languange id for language en for bar mail
     * @return string en langauage id
     */
    public function getLangIDforBar()
    {
        $id =  Db::getInstance()->ExecuteS("SELECT `id_lang` FROM "._DB_PREFIX_."lang WHERE `iso_code` = 'en'");
        return $id[0]['id_lang'];
    }

    /**
     * Get Syncdate for Products
     * @param mixed $presta_ref PrestaShop orde reference
     * @return array sync products
     */
    public function getYellowCubeSyncForProduct($presta_ref)
    {
        $value =  Db::getInstance()->ExecuteS(
            "SELECT * FROM "._DB_PREFIX_."yellowcube_product_sync WHERE `presta_reference` = '".(string)$presta_ref."'"
        );
        return $value;
    }

    /**
     * Check if a Product ID is an attribute
     * @param mixed $product_reference product reference
     * @return bool TRUE: Product is an attribute || FALSE: Product is not an attribute
     */
    public function checkIfProductAttribute($product_reference)
    {
        $value = Db::getInstance()->ExecuteS(
            "SELECT * FROM "._DB_PREFIX_."product WHERE `reference` = '".(string)$product_reference."'"
        );

        if ($value == null) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Get Product ID per reference
     * @param mixed $product_reference Product reference
     * @return string product reference
     */
    public function getProductIdPerReference($product_reference)
    {
        $value = Db::getInstance()->ExecuteS(
            "SELECT `id_product` FROM "._DB_PREFIX_."product WHERE `reference` = '".(string)$product_reference."'"
        );

        return $value[0]['id_product'];
    }

    /**
     * Get lang id for de, fr, it and en language
     * @return array langauge ids
     */
    public function getLangArrayForArt()
    {
        $langs = Db::getInstance()->ExecuteS(
            "SELECT `id_lang`, `iso_code` FROM "._DB_PREFIX_."lang WHERE `iso_code` IN ('de', 'fr', 'it', 'en') "
        );

        return $langs;
    }

    /**
     * Get the product data for a variant
     * @param mixed $product_reference
     * @return mixed productvariant data
     */
    public function getProductVariantData($product_reference)
    {
        $statement = "SELECT `id_product_attribute`, `id_product`, `ean13` FROM "
            ._DB_PREFIX_."product_attribute WHERE `reference` = '"
            .(string)$product_reference."'";
        return Db::getInstance()->ExecuteS($statement);
    }

    /**
     * get attribute id
     * @param mixed $productAttributeID product attribute id
     * @return string attribute id
     */
    public function getProductAttribute($productAttributeID)
    {
        $statement = "SELECT `id_attribute` FROM "._DB_PREFIX_.
            "product_attribute_combination WHERE `id_product_attribute` = "
            .(int)$productAttributeID." ORDER BY `id_attribute`";
        return Db::getInstance()->ExecuteS($statement);
    }

    /**
     * Insert yellowcube sync date
     * @param string $reference product reference
     */
    public function insertYellowCubeSync($reference)
    {
        $statement = "INSERT INTO "._DB_PREFIX_."yellowcube_product_sync(
            `presta_reference`, `timpestamp`) VALUES ('".(string)$reference."','".date("Y-m-d H:i:s")."')";
        Db::getInstance()->Execute($statement);
    }

    /**
     * Update yellowcube sync date
     * @param string $reference product reference
     */
    public function updateYellowCubeSync($reference)
    {
        $statement = "UPDATE "._DB_PREFIX_."yellowcube_product_sync SET `presta_reference`='"
            .(string)$reference."',`timpestamp`='"
            .date("Y-m-d H:i:s")."' WHERE `presta_reference` = '".(string)$reference."'";
        Db::getInstance()->Execute($statement);
    }

    /**
     * Get update date for product
     * @param mixed $reference product reference
     * @return DateTime the update time
     */
    public function getUpdateDateForProduct($reference)
    {
        $date = Db::getInstance()->ExecuteS(
            "SELECT `date_upd` FROM "._DB_PREFIX_."product WHERE `reference` = '".(string)$reference."'"
        );

        return DateTime::createFromFormat("Y-m-d H:i:s", $date[0]['date_upd']);
    }

    /**
     * Get update date for product variant
     * @param mixed $reference product reference
     * @return DateTime the update time
     */
    public function getUpdateDateForProductVariant($reference)
    {
        $data = $this->getProductVariantData($reference);

        $date = Db::getInstance()->ExecuteS(
            "SELECT `date_upd` FROM "._DB_PREFIX_."product WHERE `id_product` = ".(int)$data[0]['id_product']
        );

        return DateTime::createFromFormat("Y-m-d H:i:s", $date[0]['date_upd']);
    }

    /**
    * Update the bar with yellowcube ART answer
    * @param mixed $code yellowcube state code
     * @param int $state yellowcubestate enum
    * @param mixed $message yellowcube message
    * @param mixed $ref product ref
    * @param mixed $ycref yellowcube request id
    */
    public function updateBARWithArtAnswer($code, $state, $message, $ref, $ycref)
    {
        $statement = 'UPDATE '._DB_PREFIX_.'yellowcube_bar SET `statuscode`='
            .(int)$code.',`stateID`='.(int)$state.',`statustext`="'
            .(string)$message.'", `yc_reference`="'
            .(string)$ycref.'" WHERE `presta_reference` = "'.(string)$ref.'"';
        Db::getInstance()->Execute($statement);
    }

    /**
    * get the art request with statecode 10
    * @return array object with presta_reference and yc_reference
    */
    public function getPendingART()
    {
        return Db::getInstance()->ExecuteS(
            "SELECT `presta_reference`, `yc_reference` FROM "._DB_PREFIX_."yellowcube_bar WHERE `statuscode` = 10"
        );
    }

    // LOCK

    /**
    * Get existing lock for order
    * @param string $orderReference the PrestaShop reference
    * @return array lock data
    */
    public static function getLock($orderReference)
    {
        return Db::getInstance()->ExecuteS(
            "SELECT * FROM "._DB_PREFIX_."yellowcube_lock WHERE `presta_reference` ='".(string)$orderReference."'"
        );
    }

    /**
    * Create a new lock for a new WAB transfer
    * @param string $orderReference the order reference
    * @param string $key Lock-Key
    */
    public static function createLock($orderReference, $key)
    {
        $statement = "INSERT INTO "._DB_PREFIX_."yellowcube_lock(`presta_reference`, `order_key`, `timestamp`)
            VALUES ('".(string)$orderReference."','".(string)$key."','".date("Y-m-d H:i:s:ms")."')";
        return Db::getInstance()->Execute($statement);
    }

    /**
    * delete existing lock
    * @param mixed $orderReference order reference
    */
    public static function deleteLock($orderReference)
    {
        return Db::getInstance()->Execute(
            "DELETE FROM "._DB_PREFIX_."yellowcube_lock WHERE `presta_reference` = '".(string)$orderReference."'"
        );
    }

    /**
    * Get old not deleted locks
    * @return mixed
    */
    public static function getOldLocks()
    {
        return Db::getInstance()->ExecuteS(
            "SELECT * FROM "._DB_PREFIX_."yellowcube_lock where timestamp (`timestamp`) < NOW() - INTERVAL 5 MINUTE"
        );
    }

    /**
    * Checks if an ProductReference still are used
    * @param mixed $prestaReference The ProdutcReference
    * @return bool TRUE: ProductReference still used || FALSE: ProductReference unused
    */
    public static function checkIfPrestaReferenceStillAvailable($productReference)
    {
        $value = Db::getInstance()->ExecuteS(
            "SELECT * FROM "._DB_PREFIX_."product WHERE `reference` = '".(string)$productReference."'"
        );
        if ($value != null) {
            return true;
        } else {
            $valueVariant = Db::getInstance()->ExecuteS(
                "SELECT * FROM "._DB_PREFIX_."product_attribute WHERE `reference` = '".(string)$productReference."'"
            );
            if ($valueVariant != null) {
                return true;
            }
        }
        return false;
    }

    /**
    * Remove unused ProductReference Products
    * @param mixed $productReference The ProdutcReference
    */
    public static function removeProductFromBar($productReference)
    {
        Db::getInstance()->Execute(
            "DELETE FROM "._DB_PREFIX_."yellowcube_bar WHERE `presta_reference` = '".(string)$productReference."'"
        );

        $statement = "DELETE FROM "._DB_PREFIX_.
            "yellowcube_product_sync WHERE `presta_reference` = '".(string)$productReference."'";
        return Db::getInstance()->Execute($statement);
    }

    /**
     * Select all YellowCube States
     * @return mixed
     */
    public static function getYellowCubeStates()
    {
        return Db::getInstance()->ExecuteS("SELECT * FROM "._DB_PREFIX_."yellowcube_state");
    }
}
