<?php
/**
 * PrestaShopGateway short summary.
 *
 * PrestaShopGateway description.
 *
 * Entwicklung: 	Zangger Websolutions
 * Web:			www.ihrewebagentur.ch
 * Projekt:		www.yellowcube.ihrewebagentur.ch
 * @version:		4.0
 */

include_once(dirname(__FILE__).'/../../libraries/YellowCubeService/yellowcubesoap.php');
include_once(dirname(__FILE__).'/prestashopgateway.php');
include_once(dirname(__FILE__).'/../orderposition.php');
include_once(dirname(__FILE__).'/../orderdata.php');
include_once(dirname(__FILE__).'/../const.php');
include_once(dirname(__FILE__).'/../common.php');

class YellowCubeGateway
{

    public function __construct()
    {
    }

    /**
    * Send YellowCube WAB Request to YellowCube
    * @param OrderData $orderdata he OrderData Object with the Order specific Data
    */
    public function sendWAB($orderdata)
    {
        try {
            // Call the YC Function
            $response = $this->sendWABRequest($orderdata);
            return $response;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
    * Update the YellowCube Order with Status 10
    * @param string $ycref The YellowCube request id

    */
    public function updateYellowCubeWAB($ycref)
    {
        try {
            $response = $this->sendWABUpdate($ycref);
            return $response;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
    * Update the YellowCube order with the trackingnumber(WAR) from YellowCube
    * @param string $ref The PrestaShop order reference
    * @return mixed YellowCube soap response
    * Update 05/2024: Info SOAP Errors
    */
    public function getWAR($ref)
    {
        try {
            // Call the YC Function
            $response = $this->sendWARRequest($ref);
            return $response;
        } 
		catch (Exception $e) {
            $errorText = htmlentities($e->faultstring);
            Logger::addLog("Fehler: $e / Update the YellowCube order: FaultString: ".$errorText, 4);
        }

    }

    /**
    * Get Bar from YellowCube
    * @return mixed YellowCube soap response
    * Update 05/2024: Errors SOAP
    */
    public function getBAR()
    {
        try {
            // Call the YC Function
            $response = $this->sendBARRequest();
            return $response;
        } 	
			// Error?
		catch (Exception $e) {
            $errorText = htmlentities($e->faultstring);
            Logger::addLog("Fehler Get Bar from YellowCube: $e / FaultString: ".$errorText, 4);
        }
    }

    /**
    * Send an ART to Yellowcube
    * @param Art $article The art object
    */
    public function sendART($article)
    {
        // Create Presta Instance for inserting in the DB
        $prestaGateway = new PrestaShopGateway();
        try {
            // Call the YC Function
            $response = $this->sendARTRequest($article);
            $num = $response->StatusCode;
            switch ($num) {
                case 10:
                    $prestaGateway->updateBARWithArtAnswer(
                        $response->StatusCode,
                        YellowCubeState::PENDING,
                        "",
                        $article->articleNo,
                        $response->Reference
                    );
                    break;
                case 100:
                    $prestaGateway->updateBARWithArtAnswer(
                        $response->StatusCode,
                        YellowCubeState::FINISHED,
                        "",
                        $article->articleNo,
                        $response->Reference
                    );
                    break;
                default:
                    $prestaGateway->updateBARWithArtAnswer(
                        $response->StatusCode,
                        YellowCubeState::ERROR,
                        "",
                        $article->articleNo,
                        $response->Reference
                    );
                    break;
            }
        } catch (Exception $e) {
            $errorText = htmlentities($e->faultstring);
            Logger::addLog("Error: $e / Error sending ART FaultString: ".$errorText, 4);
            $prestaGateway->updateBARWithArtAnswer(
                9999,
                YellowCubeState::ERROR,
                $errorText,
                $article->articleNo,
                ""
            );
        }
    }


    /**
    * Get updatet ART Status from YellowCube
    * @param string $presta_ref PrestaShop reference from yellowcube_bar table
    * @param string $ycreference YellowCube reference from yellowcube_bar table
    */
    public function updateART($presta_ref, $ycreference)
    {
        // Create Presta Instance for inserting in the DB
        $prestaGateway = new PrestaShopGateway();
        try {
            // Call the YC Function
            $response = $this->sendARTUpdate($ycreference);
            $num = $response->StatusCode;
            switch ($num) {
                case 10:
                    $prestaGateway->updateBARWithArtAnswer(
                        $response->StatusCode,
                        YellowCubeState::PENDING,
                        "",
                        $presta_ref,
                        $response->Reference
                    );
                    break;
                case 100:
                    $prestaGateway->updateBARWithArtAnswer(
                        $response->StatusCode,
                        YellowCubeState::FINISHED,
                        "",
                        $presta_ref,
                        $response->Reference
                    );
                    break;
                default:
                    $prestaGateway->updateBARWithArtAnswer(
                        $response->StatusCode,
                        YellowCubeState::ERROR,
                        "",
                        $presta_ref,
                        $response->Reference
                    );
                    break;
            }

            $prestaGateway->updateYellowCubeSync($presta_ref);
        } catch (Exception $e) {
            $errorText = htmlentities($e->faultstring);
            Logger::addLog("Error Updating ART FaultString: ".$errorText, 4);
            $prestaGateway->updateBARWithArtAnswer(
                $e->detail->GEN_Response->StatusCode,
                YellowCubeState::ERROR,
                $errorText,
                $presta_ref,
                ""
            );
        }
    }

    /**
    * Send YellowCube ART Request to YellowCube with Certification WSSE SoapClient
    * @param mixed $article The art object with the order specific data
    * @return mixed YellowCube soap response
    */
    private function sendARTRequest($article)
    {
        // Increase the Timeout
        set_time_limit(10);
        $art = $this->createArtRequest($article);
        $cert = dirname(__FILE__)."/../../cert/".Constants::CERTIFICATIONNAME;
        // instance the soap client
        $client = new YellowCubeSoap(Constants::WSDLURL, array('local_cert' => $cert));
        // Call the YC Function
        $response = $client->__soapCall("insertArticleMasterData", array($art));
        return $response;
    }

    /**
    * Send YellowCube art update request to YellowCube to update pedning request
    * @param string $ycreference yellowcube art id
    * @return mixed YellowCube soap response
    * Update 05/2024: More timeout
    */

    private function sendARTUpdate($ycreference)
    {
        // Increase the Timeout
        set_time_limit(10);
        $art = $this->createARTUpdate($ycreference);
        $cert = dirname(__FILE__)."/../../cert/".Constants::CERTIFICATIONNAME;
        // instance the soap client
        $client = new YellowCubeSoap(Constants::WSDLURL, array('local_cert' => $cert));
        // Call the YC Function
        $response = $client->__soapCall("getInsertArticleMasterDataStatus", array($art));
        return $response;
    }


    /**
    * Send YellowCube WAB Request to YellowCube with Certification WSSE SoapClient
    * @param OrderData $orderData  The OrderData object with the order specific data
    * @return mixed YellowCube soap response
    */
    private function sendWABRequest($orderData)
    {
        // Increase the Timeout
        set_time_limit(20);
        $wabOrder = $this->createYCCustomerOrder($orderData);
        $cert = dirname(__FILE__)."/../../cert/".Constants::CERTIFICATIONNAME;
        // instance the soap client
        $client = new YellowCubeSoap(Constants::WSDLURL, array('local_cert' => $cert));
        // Call the YC Function
        $response = $client->__soapCall("createYCCustomerOrder", array($wabOrder));
        return $response;
    }

    /**
    * Update the YellowCube Order with Status 10 with Certification WSSE SoapClient
    * @param string $ycref The YellowCube Order Id
    * @return mixed YellowCube soap response
    */
    private function sendWABUpdate($ycref)
    {
        $wabUpdate = $this->createYCUpdateOrder($ycref);
        $cert = dirname(__FILE__)."/../../cert/".Constants::CERTIFICATIONNAME;
        // instance the soap client
        $client = new YellowCubeSoap(Constants::WSDLURL, array('local_cert' => $cert));
        // Call the YC Function
        $response = $client->__soapCall("getYCCustomerOrderStatus", array($wabUpdate));
        return $response;
    }

    /**
    * Update the YellowCube order with the trackingnumber(WAR) from YellowCube with Certification WSSE SoapClient
    * @param mixed $ref the reference
    * @return mixed YellowCube soap response
    */
    private function sendWARRequest($ref)
    {
        // Increase the Timeout
        set_time_limit(20);
        $war = $this->createWARRequest($ref);
        $cert = dirname(__FILE__)."/../../cert/".Constants::CERTIFICATIONNAME;
        // instance the soap client
        $client = new YellowCubeSoap(Constants::WSDLURL, array('local_cert' => $cert));
        // Call the YC Function
        $response = $client->__soapCall("getYCCustomerOrderReply", array($war));
        return $response;
    }



    /**
    * Get bar from YellowCube
    * @return mixed YellowCube soap response
    * Update 05/24: Create more time and Errors
    */


// Error handling (consider incorporating for robustness)
function handleError($message) {
    echo "Error: $message\n";
    exit(1);
}


private function sendBARRequest()
{
    $timeout = 120;

    $bar = $this->createBARRequest();
    $cert = dirname(__FILE__) . "/../../cert/" . Constants::CERTIFICATIONNAME;

    try {
        // Instance the soap client, potentially with certificate options
        $client = new YellowCubeSoap(Constants::WSDLURL, ['local_cert' => $cert]);

        // Call the YC Function with error handling
        $response = $client->__soapCall("getInventory", array($bar));
        return $response;
    } 
	// Error?
		catch (Exception $e) {
        Logger::addLog("Error sendBARRequest: $e / Error during WSDL connection", 4);
        return null; // Or throw an exception if appropriate
    }
}




    /**
    * Create correct formatted timestamp (Long Value)
    * @return string formatted date
    */
    private function getDatetimeNow()
    {
        // Create Timestamps
        $datearray = date_parse(date("Y-m-d H:i:s"));
        // Build Return Date Value and format to 2 dezimal
        $result = $datearray['year'].sprintf(
            "%02d%02d%02d%02d%02d",
            $datearray['month'],
            $datearray['day'],
            $datearray['hour'],
            $datearray['minute'],
            $datearray['second']
        );
        return $result;
    }

    /**
    * Create the YellowCube WAB request
    * @param OrderData $orderdata The orderData object with the order specific Data
    * @return array[] An associative arra which represent the request
    */
    private function createYCCustomerOrder($orderdata)
    {
        $orderpostion_array = array();
        $orderpostion_array['Position'] = array();
        // Process to Create Position assoziatives Array
        foreach ($orderdata->OrderPosition as $value) {
            $order_position_current =array('PosNo' => $value->PosNo,
                'ArticleNo' => $value->ArticleNo,
                'Plant' => $value->Plant,
                'Quantity' => $value->Quantity,
                'QuantityISO' => $value->QuantityISO
            );
            array_push($orderpostion_array['Position'], $order_position_current);
        }

        //PDF
        $docXML = '';
        if ($orderdata->OrderDocumentsFlag == 1) {
            $docXML = array('OrderDocumentsFlag' => $orderdata->OrderDocumentsFlag,
                'Docs' => array('DocType' => Constants::DOCTYPE,
                    'DocMimeType' => Constants::DOCMIMETYPE,
                    'DocStream' => $orderdata->DocStream,)
                );
        } else {
            $docXML = array(
            'OrderDocumentsFlag' => $orderdata->OrderDocumentsFlag,
            'Docs' => array()
            );
        }

        // Build WAB Object
        $params = array('ControlReference' => $this->createControlReference('WAB'),
            'Order' => array('OrderHeader' => array('DepositorNo' => $orderdata->DepositorNo,
                                'CustomerOrderNo' => $orderdata->CustomerOrderNo,
                                'CustomerOrderDate' => $orderdata->CustomerOrderDate,
                                 ),
                        'PartnerAddress' => array('Partner' => array('PartnerType' => $orderdata->PartnerType,
                                                    'PartnerNo' => $orderdata->PartnerNo,
                                                    'PartnerReference' => $orderdata->PartnerReference,
                                                    'Name1' => $orderdata->Name1,
                                                    'Name2' => $orderdata->Name2,
                                                    'Name3' => $orderdata->Name3,
                                                    'Name4' => $orderdata->Name4,
                                                    'Street' => $orderdata->Street,
                                                    'CountryCode'=> $orderdata->CountryCode,
                                                    'ZIPCode' => $orderdata->ZIPCode,
                                                    'City' => $orderdata->City,
                                                    'LanguageCode' => $orderdata->LanguageCode,
                                                ),
                                            ),
                                            'ValueAddedServices' => array('AdditionalService' => array(
                                                'BasicShippingServices' => $orderdata->BasicShippingServices,
                                                'AdditionalShippingServices'=> $orderdata->AdditionalShippingService,
                                                ),
                                            ),
                                            'OrderPositions' => $orderpostion_array,
                                            'OrderDocuments' => $docXML,
                                        )
                                    );
        return $params;
    }

    /**
    * Create the YellowCube WAB update request
    * @param mixed $ycreference The YellowCube order id
    * @return array An associative arra which represent the request
    */
    private function createYCUpdateOrder($ycreference)
    {
        // Build WAB Object
        $params = array(
            'ControlReference' => $this->createControlReference('WAB'),
            'Reference' => $ycreference,
        );
        return $params;
    }

    /**
    * Create the YellowCube WAR update request
    * @param mixed $ref The PrestaShop order reference
    * @return array An associative arra which represent the request
    */
    private function createWARRequest($ref)
    {
        // Build WAR Object
        $params = array(
            'ControlReference' => $this->createControlReference('WAR'),
            'CustomerOrderNo' => $ref,
        );
        return $params;
    }

    /**
    * Create the BAR request
    * @return array[] An associative arra which represent the request
    */
    private function createBARRequest()
    {
        // Build WAR Object
        $params = array(
            'ControlReference' => $this->createControlReference('BAR'));
        return $params;
    }

    /**
    * Create the ART request
    * @param Art $article The article object
    * @return array[] An associative arra which represent the request
    */
    private function createArtRequest($article)
    {
        // Build ART Object
        $params = array('ControlReference' => $this->createControlReference('ART'),
            'ArticleList' => array('Article' => array('ChangeFlag' => $article->changeFlag,
                    'DepositorNo' => $article->depositorNo,
                    'PlantID' => $article->plantID,
                    'ArticleNo' => $article->articleNo,
                    'BaseUOM' => $article->alternateUnitISO,
                    'NetWeight' => array('ISO' => 'KGM',
                        '_' => $article->netWeight,
                        ),
                    'UnitsOfMeasure' => array('EAN' => array('EANType' => Constants::EANTYPE,
                            '_' => $article->ean,
                        ),
                        'AlternateUnitISO' => $article->alternateUnitISO,
                        'GrossWeight' => array('ISO' => Constants::ISOWEIGHT,
                            '_' => $article->grossWeight,
                        ),
                        'Length' => array('ISO' => Constants::ISOLENGTH,
                            '_' => $article->length,
                        ),
                        'Width' => array('ISO' => Constants::ISOWIDTH,
                            '_' => $article->width,
                        ),
                        'Height' => array('ISO' => Constants::ISOHEIGHT,
                            '_' => $article->height,
                        ),
                        'Volume' => array('ISO' => Constants::ISOVOLUME,
                            '_' => $article->volume,
                        ),
                    ),
                    'ArticleDescriptions' => array('ArticleDescription' =>  array(
                        'ArticleDescriptionLC' => 'de',
                            '_' => array_key_exists('de', $article->articleDescription)
                            ? $article->articleDescription['de'] : '-',
                        ),
                        'ArticleDescriptionFR' =>  array(
                            'ArticleDescriptionLC' => array_key_exists('fr', $article->articleDescription)
                            ? $article->articleDescription['fr'] : '-',
                            '_' => 'fr',
                        ),
                    ),
                )
            )
        );
        return $params;
    }

    /**
    * Create the ART update request
    * @param mixed $ycreference YellowCube request reference
    * @return array An associative arra which represent the request
    */
    private function createARTUpdate($ycreference)
    {
        // Build WAB Object
        $params = array(
            'ControlReference' => $this->createControlReference('ART'),
            'Reference' => $ycreference,
        );
        return $params;
    }

    /**
    * Creates the controlreference header for YellowCube request
    * @param string $type  ART, WAB or BAR
    * @return array controlreference header
    */
    private function createControlReference($type)
    {
        $dateTimeStamp = new SoapVar((float)$this->getDatetimeNow(), XSD_UNSIGNEDLONG);
        return array('Type' => $type,
            'Sender' => PrestaShopGateway::getConfig(Constants::SENDER),
            'Receiver' => PrestaShopGateway::getConfig(Constants::RECEIVER),
            'Timestamp' => $dateTimeStamp,
            'OperatingMode' => PrestaShopGateway::getConfig(Constants::OPERATINGMODE),
            'Version' => '1.0',
        );
    }
}
