<?php
/**
 * YellowCube PrestaShop Modul
 *
 * Entwicklung: 	Zangger Websolutions
 * Web:			www.ihrewebagentur.ch
 * Projekt:		www.yellowcube.ihrewebagentur.ch
 * @version:		4.0
 */


include_once(dirname(__FILE__).'/dal/prestashopgateway.php');

class Lock
{
    /**
     * creates a key
     * @return string the lock key
     */
    private static function createKey()
    {
        $randomString = Tools::substr(
            str_shuffle("0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"),
            0,
            15
        );

        return $randomString;
    }

    /**
     * Check if a lock with this order reference already exist
     * @param string $orderReference the order reference
     * @return bool TRUE: Lock exist || FALSE: Lock doesn't exist
     */
    public static function checkExistingLocks($orderReference)
    {
        if (PrestaShopGateway::getLock($orderReference) == null) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Create a new lock
     * @param mixed $orderReference order reference
     * @return string the generated key
     */
    public static function createLock($orderReference)
    {
        $key = Lock::createKey();

        PrestaShopGateway::createLock($orderReference, $key);

        return $key;
    }

    /**
     * Check if the key is correct
     * @param string $orderReference the order reference
     * @param string $key the key
     * @return bool TRUE: key is correct || FALSE: key is incorrect
     */
    public static function checkLockKey($orderReference, $key)
    {
        $lockRecord = PrestaShopGateway::getLock($orderReference);

        if ($lockRecord == null) {
            return false;
        } else {
            if ($lockRecord[0]['order_key'] == $key) {
                return true;
            }
        }

        return false;
    }

    /**
     * Delete existing lock
     * @param mixed $orderReference order reference
     */
    public static function removeLock($orderReference)
    {
        PrestaShopGateway::deleteLock($orderReference);
    }

    /**
     * Remove all old locks
     */
    public static function removeOldLocks()
    {
        $oldLocks = PrestaShopGateway::getOldLocks();

        if ($oldLocks != null) {
            foreach ($oldLocks as $lock) {
                Lock::removeLock($lock['presta_reference']);
            }
        }
    }
}
