<?php
/**
 * wabservice short summary.
 *
 * YellowCube PrestaShop Modul
 *
 * Entwicklung: 	Zangger Websolutions
 * Web:			www.ihrewebagentur.ch
 * Projekt:		www.yellowcube.ihrewebagentur.ch
 * @version:		4.0
 */


include_once(dirname(__FILE__).'/../dal/prestashopgateway.php');
include_once(dirname(__FILE__).'/../dal/yellowcubegateway.php');
include_once(dirname(__FILE__).'/../orderposition.php');
include_once(dirname(__FILE__).'/../orderdata.php');
include_once(dirname(__FILE__).'/../const.php');
include_once(dirname(__FILE__).'/../lock.php');
include_once(dirname(__FILE__).'/../enum/artchangeflag.php');
include_once(dirname(__FILE__).'/../enum/yellowcubestate.php');
include_once(dirname(__FILE__).'/../dal/mailprovider.php');

class WabService
{
    /**
     * Send WAB to YellowCube with key
     * @param mixed $orderdata WAB order Data
     * @param mixed $key WAB Order key
     */
    public static function insertWithKey($orderdata, $key)
    {
        try {
            if (Lock::checkLockKey($orderdata->CustomerOrderNo, $key)) {
                WabService::Insert($orderdata);
                Lock::removeLock($orderdata->CustomerOrderNo);
            } else {
                Logger::addLog("Lock and Key doesn't Match on Order: ".$orderdata->CustomerOrderNo, 4);
            }
        } catch (Exception $e) {
            $message = htmlentities($e->getMessage());
            Lock::removeLock($orderdata->CustomerOrderNo);
            Logger::addLog("Failed to send Order with Key to YC - Message: ".$message, 4);
        }
    }

    /**
     * Send WAB to YellowCube
     * @param mixed $orderdata WAB order Data
     */
    public static function insert($orderdata)
    {
        try {
            $yellowGateway = new YellowCubeGateway();
            $response = $yellowGateway->sendWAB($orderdata);
            $num = $response->StatusCode;
            switch ($num) {
                case 10:
                    PrestaShopGateway::insertYellowCubeOrder(
                        $response->Reference,
                        $orderdata->CustomerOrderNo,
                        $response->EventTimestamp,
                        $response->StatusCode,
                        YellowCubeState::PENDING,
                        $response->StatusText
                    );
                    break;
                case 100:
                    PrestaShopGateway::insertYellowCubeOrder(
                        $response->Reference,
                        $orderdata->CustomerOrderNo,
                        $response->EventTimestamp,
                        $response->StatusCode,
                        YellowCubeState::ACCEPTED,
                        $response->StatusText
                    );
                    break;
                default:
                    PrestaShopGateway::insertYellowCubeOrder(
                        $response->Reference,
                        $orderdata->CustomerOrderNo,
                        $response->EventTimestamp,
                        $response->StatusCode,
                        YellowCubeState::ERROR,
                        $response->StatusText
                    );
                    break;
            }
        } catch (Exception $e) {
            $message = htmlentities($e->getMessage());
            PrestaShopGateway::insertYellowCubeOrder(
                "",
                $orderdata->CustomerOrderNo,
                "",
                9999,
                YellowCubeState::ERROR,
                ""
            );
            Logger::addLog("Error Send WAB with ID: ".$orderdata->CustomerOrderNo." Message: ".$message, 4);
            throw $e;
        }
    }

    /**
     * Send a WAB order again to yellowcube when order had an error
     * @param mixed $orderdata Order data
     * @param mixed $id existing order id
     */
    public static function retryInsert($orderdata, $id)
    {
        try {
            $yellowGateway = new YellowCubeGateway();
            $response = $yellowGateway->sendWAB($orderdata);
            $num = $response->StatusCode;

            switch ($num) {
                case 10:
                    PrestaShopGateway::updateYellowCubeOrder(
                        $id,
                        $response->Reference,
                        $response->EventTimestamp,
                        $response->StatusCode,
                        YellowCubeState::PENDING,
                        $response->StatusText
                    );
                    break;
                case 100:
                    PrestaShopGateway::updateYellowCubeOrder(
                        $id,
                        $response->Reference,
                        $response->EventTimestamp,
                        $response->StatusCode,
                        YellowCubeState::ACCEPTED,
                        $response->StatusText
                    );
                    break;
                default:
                    PrestaShopGateway::updateYellowCubeOrder(
                        $id,
                        $response->Reference,
                        $response->EventTimestamp,
                        $response->StatusCode,
                        YellowCubeState::ERROR,
                        $response->StatusText
                    );
                    break;
            }
        } catch (Exception $e) {
            $message = htmlentities($e->getMessage());
            PrestaShopGateway::setYellowCubeOrderToError($id, 9999, YellowCubeState::ERROR, $message);
            Logger::addLog("Error Send WAB with ID: ".$orderdata->CustomerOrderNo." Message: ".$message, 4);
        }
    }

    /**
     * Update Code 10 orders to 100 or error
     * @param mixed $id existing order id
     * @param mixed $ycref YellowCube Order reference
     */
    public static function update($id, $ycref)
    {
        try {
            $yellowGateway = new YellowCubeGateway();
            $response = $yellowGateway->updateYellowCubeWAB($ycref);
            $num = $response->StatusCode;

            switch ($num) {
                case 10:
                    PrestaShopGateway::updateYellowCubeOrder(
                        $id,
                        $response->Reference,
                        $response->EventTimestamp,
                        $response->StatusCode,
                        YellowCubeState::PENDING,
                        $response->StatusText
                    );
                    break;
                case 100:
                    PrestaShopGateway::updateYellowCubeOrder(
                        $id,
                        $response->Reference,
                        $response->EventTimestamp,
                        $response->StatusCode,
                        YellowCubeState::ACCEPTED,
                        $response->StatusText
                    );
                    PrestaShopGateway::updateOrderStateByYCID(
                        $id,
                        PrestaShopGateway::getConfig(Constants::YELLOWCUBESHIPPEDID)
                    );
                    break;
                default:
                    PrestaShopGateway::updateYellowCubeOrder(
                        $id,
                        $response->Reference,
                        $response->EventTimestamp,
                        $response->StatusCode,
                        YellowCubeState::ERROR,
                        $response->StatusText
                    );
                    break;
            }
        } catch (Exception $e) {
            $message = $e->getMessage();
            $errorCode = $e->detail->GEN_Response->StatusCode;
            Logger::addLog("Error Update WAB with ID: ".$ycref." FaultString: ".$message, 4, null, null, null, true);
            PrestaShopGateway::setYellowCubeOrderToError($id, $errorCode, YellowCubeState::ERROR, $message);
        }
    }

    /**
     * Update Code 100 order with WAR and send email to customer
     * @param mixed $reference order reference
     */
    public static function updateWar($reference)
    {
        try {
            $yellowGateway = new YellowCubeGateway();
            $war = $yellowGateway->getWAR($reference);

            if (count((array)$war) > 0) {
                $warData = $war->WAR[0]->GoodsIssue->CustomerOrderHeader;
                $text = "YCDeliveryNo: ".$warData->YCDeliveryNo." and YCDeliveryDate: ".$warData->YCDeliveryDate;

                PrestaShopGateway::updateTrackingNumber($warData->CustomerOrderNo, $warData->PostalShipmentNo, $text);

                $order = PrestaShopGateway::getOrderByReference($warData->CustomerOrderNo);

                PrestaShopGateway::updateOrderTrackingNumber($order[0]['id_order'], $warData->PostalShipmentNo);

                $mail = new MailProvider();
                $mail->sendTrackingEmail(
                    $warData->CustomerOrderNo,
                    $order[0]['id_customer'],
                    $warData->PostalShipmentNo
                );

                PrestaShopGateway::updateOrderStateByID($order[0]['id_order'], 4);
            }
        } catch (Exception $e) {
            $message = htmlentities($e->getMessage());
            Logger::addLog("Error Updating WAR from YellowCube. Message: ".$message, 4);
        }
    }
}
