<?php
/**
 * AdminBarController short summary.
 *
 * Entwicklung: 	Zangger Websolutions
 * Web:			www.ihrewebagentur.ch
 * Projekt:		www.yellowcube.ihrewebagentur.ch
 * @version:		4.0
 */

include_once(dirname(__FILE__).'/../../classes/model/bar.php');
include_once(dirname(__FILE__).'/../../classes/dal/prestashopgateway.php');
include_once(dirname(__FILE__).'/../../yellowcube.php');

class AdminBarController extends AdminController
{
    protected $module;
    protected $statuses_array = array();

    public function __construct()
    {
        $this->module = new YellowCube();

        $this->module->refreshtBarData();

        $this->bootstrap = true;
        //$this->addRowAction('edit'); //add an edit button
        //$this->addRowAction('delete'); //add a delete button

        $this->addRowAction('sync'); //add a sync button
        $this->addRowAction('deactivate'); //add a deactivate button
        $this->explicitSelect = true;
        $this->list_no_link = true;
        $this->path = _MODULE_DIR_."yellowcube";
        $this->table = 'yellowcube_bar'; //define the main table
        $this->className = 'Bar'; //define the module entity
        $this->identifier = "id"; //the primary key

        $date = PrestaShopGateway::getConfig(Constants::LASTBARSYNC);
        $dateSync = PrestaShopGateway::getConfig(Constants::YCBARSYNC);

        $tableTitle ='BAR-Inventory-Sync: '.$date.' | BAR-Inventory-Date-Time: '.$dateSync;

        $this->toolbar_title = $tableTitle;

        $this->bulk_actions = array(
            'sync' => array(
                'text' => $this->module->l('Sync selected'),
                'icon' => 'icon-retweet',
                'confirm' => $this->module->l('Sync selected items?')
                ),
            'deactivate' => array(
                'text' => $this->module->l('Deactivate selected'),
                'icon' => 'icon-remove-sign',
                'confirm' => $this->module->l('Deactivate selected items?')
                )
        );

        $this->_select = '
            a.`id`,
            os.`color`,
            os.`name`';

        $this->_join = 'LEFT JOIN `'._DB_PREFIX_.'yellowcube_state` os ON (os.`id` = a.`stateID`)';

        $statuses = PrestaShopGateway::getYellowCubeStates();
        foreach ($statuses as $status) {
            $this->statuses_array[$status['id']] = $status['name'];
        }

        //and define the field to display in the admin table
        $this->fields_list = array(
            'id' => array(
                  'title' => $this->module->l('ID'),
                  'align' => 'left',
                  'width' => 'auto'
              ),
              'presta_reference' => array(
                  'title' => $this->module->l('Article'),
                  'align' => 'left',
                  'width' => 'auto'
              ),
              'description' => array(
                  'title' => $this->module->l('Description'),
                  'align' => 'left',
                  'width' => 'auto'
              ),
              'presta_stock' => array(
                  'title' => $this->module->l('Presta Stock'),
                  'align' => 'left',
                  'width' => 'auto'
              ),
              'yellowcube_stock' => array(
                  'title' => $this->module->l('YC Stock'),
                  'align' => 'left',
                  'width' => 'auto'
              ),
              'pending_wab' => array(
                  'title' => $this->module->l('Pending WAB'),
                  'align' => 'left',
                  'width' => 'auto'
              ),
              'difference' => array(
                  'title' => $this->module->l('Difference'),
                  'align' => 'left',
                  'width' => 'auto'
              ),
              'statuscode' => array(
                  'title' => $this->module->l('ART-StatusCode'),
                  'align' => 'left',
                  'width' => 'auto'
              ),
              'name' => array(
                'title' => $this->module->l('State'),
                'type' => 'select',
                'color' => 'color',
                'list' => $this->statuses_array,
                'filter_key' => 'os!id',
                'filter_type' => 'int',
                'order_key' => 'name'
            ),
              'statustext' => array(
                  'title' => $this->module->l('Message'),
                  'align' => 'left',
                  'width' => 'auto'
              ),
          );

        parent::__construct();
    }

    public function displaySyncLink($token, $id)
    {
        $tpl = $this->createTemplate(
            "..\..\..\../modules/yellowcube/views/templates/admin/yellowcube/helpers/list/list_action_sync.tpl"
        );

        $tpl->assign(array(
            'href' => self::$currentIndex.'&token='.$token.'&'.$this->identifier.'='.
            $id.'&sync'.$this->table.'=1',
                'action' => $this->module->l('Sync')
        ));

        return $tpl->fetch();
    }

    public function displayDeactivateLink($token, $id)
    {
        $tpl = $this->createTemplate(
            "..\..\..\../modules/yellowcube/views/templates/admin/yellowcube/helpers/list/list_action_deactivate.tpl"
        );

        $tpl->assign(array(
            'href' => self::$currentIndex.'&token='.$token.'&'
            .$this->identifier.'='.$id.'&deactivate'.$this->table.'=1',
                'action' => $this->module->l('Deactivate')
        ));

        return $tpl->fetch();
    }

    public function initToolbar()
    {
        parent::initToolbar();
        unset($this->toolbar_btn['new']);
        unset($this->toolbar_btn['export']);
    }

    /*
    public function renderForm(){


    $this->context->smarty->assign(array(
    'mymodule_controller_url' => $this->context->link->getAdminLink('AdminBar'),//give the url for ajax query
    ));

    $more = $this->module->display($path, 'view/templates/admin/bar.tpl');

    return $more.parent::renderForm();
    }*/

    /**
     * postProcess handle every checks before saving products information
     *
     * @return void
     */
    public function postProcess()
    {
        parent::postProcess(); //ruft delete auf.
    }

    public function initProcess()
    {
        parent::initProcess();
        if (Tools::getValue('sync'.$this->table)) {
            $this->display = 'sync';
            $this->action = 'sync';
        } elseif (Tools::getValue('deactivate'.$this->table)) {
            $this->display = 'deactivate';
            $this->action = 'deactivate';
        }
    }

    /**
     * Object Send
     */
    public function processSync()
    {
        if (Validate::isLoadedObject($object = $this->loadObject())) {
            return $this->module->syncART($object->presta_reference);
        }
        return false;
    }

    /**
     * Object Deactivate
     * @return bool
     */
    public function processDeactivate()
    {
        if (Validate::isLoadedObject($object = $this->loadObject())) {
            return $this->module->deactivateART($object->presta_reference);
        }
        return false;
    }

    protected function processBulkSync()
    {
        if (is_array($this->boxes) && !empty($this->boxes)) {
            $bars = Tools::getValue($this->table.'Box');

            if (is_array($bars) && (count($bars))) {
                foreach ($bars as $id) {
                    $bar = new Bar((int)$id);

                    $this->module->syncART($bar->presta_reference);
                }
            }
        } else {
            $this->errors[] = Tools::displayError('You must select at least one element to sync.');
            return false;
        }
        return true;
    }

    protected function processBulkDeactivate()
    {
        if (is_array($this->boxes) && !empty($this->boxes)) {
            $bars = Tools::getValue($this->table.'Box');

            if (is_array($bars) && (count($bars))) {
                foreach ($bars as $id) {
                    $bar = new Bar((int)$id);

                    $this->module->deactivateART($bar->presta_reference);
                }
            }
        } else {
            $this->errors[] = Tools::displayError('You must select at least one element to deactivate.');
            return false;
        }

        return true;
    }
}
