<?php
/**
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * Entwicklung: 	Zangger Websolutions
 * Web:			www.ihrewebagentur.ch
 * Projekt:		www.yellowcube.ihrewebagentur.ch
 * @version:		4.0
 */

include_once(dirname(__FILE__).'/WSASoap.php');
include_once(dirname(__FILE__).'/WSSESoap.php');
include_once(dirname(__FILE__).'/XMLSecurityKey.php');
include_once(dirname(__FILE__).'/XMLSecurityDSig.php');

use Wse\WSASoap;
use Wse\WSSESoap;
use Wse\XMLSecurityKey;



/**
 * Sets default options for SoapClient.
 *
 * Also adds better error formatting.
 *
 * @package YellowCube\Util
 */
class YellowCubeSoap extends \SoapClient
{
    /**
     * @var array Soap client options.
     */
    protected $options = array();

    /**
     * @var string Loaded certificate.
     */
    protected $certificateContent;

    /**
     * @param mixed $wsdl
     * @param array $options
     */
    public function __construct($wsdl, $options)
    {
        $defaultOptions = array(
            'soap_version' => SOAP_1_1,

            // Enables tracing for __getLastRequest() support.
            'trace' => true,

            // Always return an array for a sequence of elements.
            'features' => SOAP_SINGLE_ELEMENT_ARRAYS,
        );

        $this->options = array_merge($defaultOptions, $options);

        parent::__construct($wsdl, $this->options);

    }

    /**
     * @inheritdoc
     */
    public function __doRequest($request, $location, $action, $version, $oneWay = null)
    {
        if ($this->useCertificate()) {
            return $this->signRequest($request, $location, $action, $version, $oneWay);
        }

        return parent::__doRequest($request, $location, $action, $version, $oneWay);
    }

    /**
     * Signs the specified request.
     *
     * @param      $request
     * @param      $location
     * @param      $action
     * @param      $version
     * @param null $oneWay
     *
     * @return string
     * @throws \Exception
     */



    protected function signRequest($request, $location, $action, $version, $oneWay = null)
    {
        $doc = new \DOMDocument();
        $doc->loadXML($request);

        $wsse = new WSSESoap($doc);
        $wsse->addTimestamp();

        $key = new XMLSecurityKey(XMLSecurityKey::RSA_SHA1, array('type'=>'private'));
        $key->loadKey($this->getCertificateContent());
        $wsse->signSoapDoc($key);

        $token = $wsse->addBinaryToken($this->getCertificateContent());
        $wsse->attachTokentoSig($token);

        $signedRequest = $wsse->saveXML();
        
        /*
        $dateihandle = fopen("/../../cert/","w");
        fwrite($dateihandle, $signedRequest);*/

        return parent::__doRequest($signedRequest, $location, $action, $version, $oneWay);
    }



    /**
     * Calls the specified method on the SOAP server.
     *
     * @param string $method
     * @param string $args
     * @return mixed|void
     *
     * @throws YellowCubeException if a SOAP error occurs.
     */
    public function __call($method, $args)
    {
        try {
            return parent::__call($method, $args);
        } catch (\Exception $e) {
            $message = $e->getMessage();
            $message .= PHP_EOL . PHP_EOL . 'Request XML: ' . PHP_EOL . $this->__getLastRequest();
           // throw new YellowCubeException($message, $e->getCode(), $e);
            throw new Exception($message, $e->getCode(), $e);
            Logger::addLog("Error: $e / Calls the specified method on the SOAP server: ".$message, 4);
        }
    }

    /**
     * Returns true if a certificate should be used.
     *
     * @return mixed
     */
    protected function useCertificate()
    {
        return !empty($this->options['local_cert']);
    }



    /**
     * Returns content of the certificate passed  in 'local_cert'.
     *
     * @return string Content of the certificate passed  in 'local_cert'.
     */
    protected function getCertificateContent()
    {
        if (empty($this->certificateContent)) {
            $this->certificateContent = Tools::file_get_contents($this->options['local_cert']);
        }

        return $this->certificateContent;
    }
}
