<?php
/**
 * YellowCube PrestaShop Modul
 *
 * Entwicklung: 	Zangger Websolutions
 * Web:			www.ihrewebagentur.ch
 * Projekt:		www.yellowcube.ihrewebagentur.ch
 * @version:		4.0
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

if (strpos(dirname(__FILE__), 'yellowcube') !== false) {
    include_once(dirname(__FILE__).'/classes/dal/prestashopgateway.php');
    include_once(dirname(__FILE__).'/classes/const.php');
    include_once(dirname(__FILE__).'/classes/orderdata.php');
    include_once(dirname(__FILE__).'/classes/orderposition.php');
    include_once(dirname(__FILE__).'/classes/bll.php');
    include_once(dirname(__FILE__).'/classes/lock.php');
    include_once(dirname(__FILE__).'/classes/dal/csvreader.php');
    include_once(dirname(__FILE__).'/classes/exceptions.php');
    include_once(dirname(__FILE__).'/classes/service/wabservice.php');
}

class YellowCube extends Module
{

    public function __construct()
    {
        $this->name = 'yellowcube';
        $this->tab = 'administration';
        $this->version = '4.0.0';
        $this->author = 'Zangger Websolutions';
        $this->need_instance = 1;
        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
        $this->bootstrap = true;
        $this->module_key = 'b644a27413e4e2481cf7c8fc14f61b6f';

        parent::__construct();

        $this->displayName = $this->l('YellowCube');
        $this->description = $this->l('YellowCube connectivity module');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');

        if (!Configuration::get('YCMODULE_NAME')) {
            $this->warning = $this->l('No name provided');
        }
    }

    /**
     * Install the Plugin
     * @return bool TRUE: install successful || FALSE: Error while installing
     */
    public function install()
    {

        if (!$this->installDB()
            || !Configuration::updateValue('YCMODULE_NAME', 'yellowcube')
            || !$this->installConfig()
            || !$this->installModuleTab(
                'AdminBar',
                array(1 => 'YellowCube Inventory'),
                Tab::getIdFromClassName(Constants::TABPARENTNAME)
            )
            || !$this->installModuleTab(
                'AdminWab',
                array(1 => 'YellowCube'),
                Tab::getIdFromClassName(Constants::TABPARENTNAMEORDERS)
            )
            ) {
            Logger::addLog('YellowCube: Installing Database or Tabs failed', 4);
            return false;
        }

        return parent::install() &&
            $this->registerHook('displayOrderConfirmation') &&
            $this->registerHook('actionOrderStatusUpdate');
    }

    /**
     * Deinstall the Plugin
     * @return bool TRUE: deinstall successful || FALSE: Error while deinstalling
     */
    public function uninstall()
    {
        $this->uninstallDB();
        $this->uninstallModuleTab('AdminBar');
        $this->uninstallModuleTab('AdminWab');
        Configuration::deleteByName('YCMODULE_NAME');

        if (!$this->unregisterHook('displayOrderConfirmation') ||
        !$this->unregisterHook('actionOrderStatusUpdate')) {
            Logger::addLog('YellowCube: unregisterHook failed', 4);
            return false;
        }

        if (!parent::uninstall()) {
            Logger::addLog('YellowCube: uninstall failed', 4);
            return false;
        }

        return true;
    }

    /**
     * Installs a new Modul Backend Tab
     * @param mixed $tabClass Tabclass for Controller
     * @param mixed $tabName Tab Name
     * @param mixed $idTabParent What the Parent Tab is
     * @return bool TRUE sucessfull
     */
    private function installModuleTab($tabClass, $tabName, $idTabParent)
    {
        $tab = new Tab();
        $tab->name = $tabName;
        $tab->class_name = $tabClass;
        $tab->module = $this->name;
        $tab->id_parent = $idTabParent;
        if (!$tab->save()) {
            return false;
        }
        return true;
    }

    /**
     * Uninstall a Module-Tab
     * @param mixed $tabClass Tabclass
     * @return bool TRUE sucessfull
     */
    private function uninstallModuleTab($tabClass)
    {
        $idTab = Tab::getIdFromClassName($tabClass);
        if ($idTab != 0) {
            $tab = new Tab($idTab);
            $tab->delete();
            return true;
        }
        return false;
    }


    /**
     * Install the DB
     * @return bool install successful or not
     */
    public function installDB()
    {
        // Creation of the tables in a database
        $result = true;
        $queries = array();

        $queries[] = 'CREATE TABLE IF NOT EXISTS '._DB_PREFIX_.'yellowcube_state (
            `id` INT UNSIGNED NOT NULL AUTO_INCREMENT,
            `name` VARCHAR(100),
            `color` VARCHAR(32),
            PRIMARY KEY (`id`),
            KEY `id` (`id`)
        ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8;';

        $queries[] = 'CREATE TABLE IF NOT EXISTS '._DB_PREFIX_.'yellowcube_configuration (
            `id_yellowcube_configuration` INT UNSIGNED NOT NULL AUTO_INCREMENT,
            `name` VARCHAR(255),
            `value` VARCHAR(500),
            PRIMARY KEY (`id_yellowcube_configuration`),
            KEY `name` (`name`)
        ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8;';

        $queries[] = 'CREATE TABLE IF NOT EXISTS '._DB_PREFIX_.'yellowcube_orders (
            `id` INT UNSIGNED NOT NULL AUTO_INCREMENT,
            `yc_reference` VARCHAR(100),
            `presta_reference` VARCHAR(100),
            `timestamp` datetime,
            `statuscode` int,
            `stateID` int UNSIGNED NOT NULL,
            `statustext` VARCHAR(255),
            `tracking` VARCHAR(2500),
            PRIMARY KEY (`id`),
            FOREIGN KEY (stateID) REFERENCES '._DB_PREFIX_.'yellowcube_state(id),
            KEY `id` (`id`)
        ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8;';

        $queries[] = 'CREATE TABLE IF NOT EXISTS '._DB_PREFIX_.'yellowcube_product_sync (
            `id` INT UNSIGNED NOT NULL AUTO_INCREMENT,
            `presta_reference` VARCHAR(100),
            `timpestamp` VARCHAR(100),
            PRIMARY KEY (`id`),
            KEY `id` (`id`)
        ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8;';

        $queries[]= 'CREATE TABLE IF NOT EXISTS '._DB_PREFIX_.'yellowcube_bar (
            `id` INT UNSIGNED NOT NULL AUTO_INCREMENT,
            `presta_reference` VARCHAR(100),
            `description` VARCHAR(1000),
            `presta_stock` VARCHAR(50),
            `yellowcube_stock` VARCHAR(50),
            `pending_wab` VARCHAR(50),
            `difference` VARCHAR(50),
            `statuscode` int,
            `statustext` VARCHAR(2500),
            `yc_reference` VARCHAR(100),
            `stateID` int UNSIGNED NOT NULL,
            PRIMARY KEY (`id`),
            FOREIGN KEY (stateID) REFERENCES '._DB_PREFIX_.'yellowcube_state(id),
            KEY `id` (`id`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8;';

        $queries[] = 'CREATE TABLE IF NOT EXISTS '._DB_PREFIX_.'yellowcube_lock (
            `presta_reference` varchar(50) not null,
            `order_key` varchar(50) not null,
            `timestamp` VARCHAR(100),
            UNIQUE (presta_reference)
        ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8;';

        $queries[] = "INSERT INTO `"._DB_PREFIX_."yellowcube_state` (`name`, `color`) VALUES (
            'Error', '#cc0000');";

        $queries[] = "INSERT INTO `"._DB_PREFIX_."yellowcube_state` (`name`, `color`) VALUES (
            'Pending', '#ff9900');";

        $queries[] = "INSERT INTO `"._DB_PREFIX_."yellowcube_state` (`name`, `color`) VALUES (
            'Accepted', '#357EB2');";

        $queries[] = "INSERT INTO `"._DB_PREFIX_."yellowcube_state` (`name`, `color`) VALUES (
            'Finished', '#339933');";

        $queries[] = "INSERT INTO `"._DB_PREFIX_."yellowcube_state` (`name`, `color`) VALUES (
            'Undefined', '#BDBDBD');";

        $queries =  array_merge($queries, $this->createStatus());

        foreach ($queries as $query) {
            $result &= Db::getInstance()->Execute($query);
        }

        if (!$result) {
            return false;
        }

        return true;
    }

    /**
     * Create the two modul states
     * @return string[] query array
     */
    private function createStatus()
    {
        $queries = array();

        $maxId = Db::getInstance()->ExecuteS(
            'SELECT `id_order_state` from '._DB_PREFIX_.'order_state ORDER BY `id_order_state` DESC LIMIT 1'
        );
        $id = (int)$maxId[0]['id_order_state'];
        $queries[] = "INSERT INTO "._DB_PREFIX_."order_state (`id_order_state`, `invoice`, `send_email`, `module_name`,
            `color`, `unremovable`, `hidden`, `logable`, `delivery`, `shipped`, `paid`, `deleted`) VALUES
            (".++$id." ,0, 0, '', '#ffff00', 0, 1, 0, 1, 0, 0, 0)";

        $languages = Language::getLanguages();
        foreach ($languages as $language) {
            $queries[] = "INSERT INTO "._DB_PREFIX_."order_state_lang (`id_order_state`, `id_lang`, `name`) VALUES
                            (".$id.", ".$language['id_lang'].", '".Constants::STATE."')";
        }

        $queries[] = "INSERT INTO "._DB_PREFIX_."order_state (`id_order_state`, `invoice`, `send_email`, `module_name`,
            `color`, `unremovable`, `hidden`, `logable`, `delivery`, `shipped`, `paid`, `deleted`) VALUES
            (".++$id." ,0, 0, '', '#ffb400', 0, 1, 0, 1, 0, 0, 0)";

        $languages = Language::getLanguages();
        foreach ($languages as $language) {
            $queries[] = "INSERT INTO "._DB_PREFIX_."order_state_lang (`id_order_state`, `id_lang`, `name`) VALUES
                            (".$id.", ".$language['id_lang'].", '".Constants::YELLOWCUBESHIPPED."')";
        }

        return $queries;
    }


    /**
     * Create the Config Values
     * @return bool true
     */
    public function installConfig()
    {
        PrestaShopGateway::insertConfig(Constants::SENDER, '');
        PrestaShopGateway::insertConfig(Constants::VERSION, '1.0');
        PrestaShopGateway::insertConfig(Constants::DEPOSITOR_NO, '');
        PrestaShopGateway::insertConfig(Constants::PARTNER_TYPE, 'WE');
        PrestaShopGateway::insertConfig(Constants::PARTNER_NO, '');
        PrestaShopGateway::insertConfig(Constants::RECEIVER, 'YELLOWCUBE');
        PrestaShopGateway::insertConfig(Constants::PLANT, '');
        PrestaShopGateway::insertConfig(Constants::QUANTITY_ISO, '');
        PrestaShopGateway::insertConfig(Constants::OPERATINGMODE, 'T');
        PrestaShopGateway::insertConfig(Constants::DELIVERYATT, '0');
        PrestaShopGateway::insertConfig(Constants::WABCRON, '0');
        PrestaShopGateway::insertConfig(Constants::YELLOWCUBESHIPPEDID, '');
        PrestaShopGateway::insertConfig(Constants::LASTBARSYNC, '');
        PrestaShopGateway::insertConfig(Constants::YCBARSYNC, '');

        PrestaShopGateway::setYellowCubeShippedID();

        return true;
    }

    /**
     * Delete DB changes when deinstall the plugin
     * @return bool deinstall db successful or not
     */
    public function uninstallDB()
    {
        $result = true;
        $queries = array();

        $stateIDArray = Db::getInstance()->ExecuteS(
            "SELECT `id_order_state` FROM "._DB_PREFIX_."order_state_lang WHERE `name` = '".Constants::STATE."'"
        );
        $state2 = PrestaShopGateway::getConfig(Constants::YELLOWCUBESHIPPEDID);

        $stateID = (int)$stateIDArray[0]['id_order_state'];

        $queries[] = "DELETE FROM "._DB_PREFIX_."order_state_lang WHERE `id_order_state` = ".$stateID;
        $queries[] = "DELETE FROM "._DB_PREFIX_."order_state WHERE `id_order_state` = ".$stateID;
        $queries[] = "DELETE FROM "._DB_PREFIX_."order_state_lang WHERE `id_order_state` = ".$state2;
        $queries[] = "DELETE FROM "._DB_PREFIX_."order_state WHERE `id_order_state` = ".$state2;
        $queries[] = 'DROP TABLE IF EXISTS`'._DB_PREFIX_.'yellowcube_configuration`';
        $queries[] = 'DROP TABLE IF EXISTS`'._DB_PREFIX_.'yellowcube_orders`';
        $queries[] = 'DROP TABLE IF EXISTS`'._DB_PREFIX_.'yellowcube_bar`';
        $queries[] = 'DROP TABLE IF EXISTS`'._DB_PREFIX_.'yellowcube_product_sync`';
        $queries[] = 'DROP TABLE IF EXISTS`'._DB_PREFIX_.'yellowcube_lock`';
        $queries[] = 'DROP TABLE IF EXISTS`'._DB_PREFIX_.'yellowcube_state`';

        foreach ($queries as $query) {
            $result &= Db::getInstance()->Execute($query);
        }
        if (!$result) {
            return false;
        }
        return true;
    }

    /**
     * Get content for configuration page in modules page
     * @return string config form
     */


    public function getHeader()
				{
           $output = '';
        $output = $this->context->smarty->fetch($this->local_path.'views/templates/admin/header-configure.tpl');
        return $output.$this->renderForm();
				}

    public function getContent()
    {
     $output = null;

        if (Tools::isSubmit('submit'.$this->name)) {
            $sender = (string)Tools::getValue(Constants::SENDER);
            $depositorno = (string)Tools::getValue(Constants::DEPOSITOR_NO);
            $partnertype = (string)Tools::getValue(Constants::PARTNER_TYPE);
            $partnerno = (string)Tools::getValue(Constants::PARTNER_NO);
            $receiver = (string)Tools::getValue(Constants::RECEIVER);
            $plant = (string)Tools::getValue(Constants::PLANT);
            $quantityiso = (string)Tools::getValue(Constants::QUANTITY_ISO);
            $operatingMode = (string)Tools::getValue(Constants::OPERATINGMODE);
            $pdf = (string)Tools::getValue(Constants::DELIVERYATT);
            $wabCron = (string)Tools::getValue(Constants::WABCRON);
            if (!$this->validateConfigValues(
                $sender,
                $depositorno,
                $partnertype,
                $partnerno,
                $receiver,
                $plant,
                $quantityiso,
                $operatingMode
            )) {
                $output .= $this->displayError($this->l('Invalid Configuration value'));
            } else {
                PrestaShopGateway::updateConfig(Constants::SENDER, $sender);
                PrestaShopGateway::updateConfig(Constants::DEPOSITOR_NO, $depositorno);
                PrestaShopGateway::updateConfig(Constants::PARTNER_TYPE, $partnertype);
                PrestaShopGateway::updateConfig(Constants::PARTNER_NO, $partnerno);
                PrestaShopGateway::updateConfig(Constants::RECEIVER, $receiver);
                PrestaShopGateway::updateConfig(Constants::PLANT, $plant);
                PrestaShopGateway::updateConfig(Constants::QUANTITY_ISO, $quantityiso);
                PrestaShopGateway::updateConfig(Constants::OPERATINGMODE, $operatingMode);
                PrestaShopGateway::updateConfig(Constants::DELIVERYATT, $pdf);
                PrestaShopGateway::updateConfig(Constants::WABCRON, $wabCron);

                $output .= $this->displayConfirmation($this->l('Settings updated'));
            }
        }

       return $output.$this->displayForm();


    }

    /**
     * Display the form for the modulconfiguration
     * @return mixed generated form
     */
    public function displayForm()
    {
        $options = array(
            array(
                'id_option' => 1,
                'name' => 'True'
            ),
            array(
                'id_option' => 0,
                'name' => 'False'
            ),
        );

        // Get default language
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');
        $fields_form = array();
        // Init Fields form array
        $fields_form[0]['form'] = array(
            'legend' => array(
                'title' => $this->l('Settings'),
            ),
            'input' => array(
                array(
                    'type' => 'text',
                    'label' => $this->l(Constants::SENDER),
                    'name' => Constants::SENDER,
                    'size' => 20,
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l(Constants::DEPOSITOR_NO),
                    'name' => Constants::DEPOSITOR_NO,
                    'size' => 20,
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l(Constants::PARTNER_NO),
                    'name' => Constants::PARTNER_NO,
                    'size' => 20,
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l(Constants::PARTNER_TYPE),
                    'name' => Constants::PARTNER_TYPE,
                    'size' => 20,
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l(Constants::RECEIVER),
                    'name' => Constants::RECEIVER,
                    'size' => 20,
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l(Constants::PLANT),
                    'name' => Constants::PLANT,
                    'size' => 20,
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l(Constants::QUANTITY_ISO),
                    'name' => Constants::QUANTITY_ISO,
                    'size' => 20,
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l(Constants::OPERATINGMODE),
                    'name' => Constants::OPERATINGMODE,
                    'size' => 1,
                    'required' => true
                ),
                array(
                  'type' => 'select',
                  'label' => $this->l(Constants::DELIVERYATT),
                  'desc' => $this->l('FALSE = disabled | TRUE = Send DeliveryPDF to YellowCube'),
                  'name' => Constants::DELIVERYATT,
                  'required' => true,
                  'options' => array(
                    'query' => $options,
                    'id' => 'id_option',
                    'name' => 'name'
                  )
                ),
                array(
                  'type' => 'select',
                  'label' => $this->l(Constants::WABCRON),
                  'desc' => $this->l('FALSE = disabled | TRUE = Use Cronjob to send WAB'),
                  'name' => Constants::WABCRON,
                  'required' => true,
                  'options' => array(
                    'query' => $options,
                    'id' => 'id_option',
                    'name' => 'name'
                  )
                ),
                array(
                    'type' => 'free',
                    'label' => 'WSDL-URL',
                    'name' => 'WSDL',
                    'size' => 50,
                    'required' => false
                )
            ),
            'submit' => array(
                'title' => $this->l('Save'),
                'class' => 'button'
            )
        );

        $helper = new HelperForm();

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

        // Language
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;        // false -> remove toolbar
        $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action = 'submit'.$this->name;
        $helper->toolbar_btn = array(
            'save' =>
            array(
                'desc' => $this->l('Save'),
                'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
                '&token='.Tools::getAdminTokenLite('AdminModules'),
            ),
            'back' => array(
                'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            )
        );

        // Load current value
        $helper->fields_value[Constants::DEPOSITOR_NO] = PrestaShopGateway::getConfig(Constants::DEPOSITOR_NO);
        $helper->fields_value[Constants::PARTNER_NO] = PrestaShopGateway::getConfig(Constants::PARTNER_NO);
        $helper->fields_value[Constants::PARTNER_TYPE] = PrestaShopGateway::getConfig(Constants::PARTNER_TYPE);
        $helper->fields_value[Constants::RECEIVER] = PrestaShopGateway::getConfig(Constants::RECEIVER);
        $helper->fields_value[Constants::PLANT] = PrestaShopGateway::getConfig(Constants::PLANT);
        $helper->fields_value[Constants::QUANTITY_ISO] = PrestaShopGateway::getConfig(Constants::QUANTITY_ISO);
        $helper->fields_value[Constants::SENDER] = PrestaShopGateway::getConfig(Constants::SENDER);
        $helper->fields_value[Constants::DELIVERYATT] = PrestaShopGateway::getConfig(Constants::DELIVERYATT);
        $helper->fields_value[Constants::WABCRON] = PrestaShopGateway::getConfig(Constants::WABCRON);
        $helper->fields_value[Constants::OPERATINGMODE] = PrestaShopGateway::getConfig(Constants::OPERATINGMODE);
        $helper->fields_value['WSDL'] = Constants::WSDLURL;

        return $helper->generateForm($fields_form);
    }

    /**
     * Validate the Config Values
     * @param mixed $sender sender value
     * @param mixed $depositorno depositor number value
     * @param mixed $partnertype partner type value
     * @param mixed $partnerno partner number value
     * @param mixed $plant plant
     * @param mixed $quantityiso quantity iso code
     * @param mixed $displaydays how long are the ordes displayed
     * @param mixed $operatingMode yellowcube operating mode
     * @return bool
     */
    public function validateConfigValues(
        $sender,
        $depositorno,
        $partnertype,
        $partnerno,
        $receiver,
        $plant,
        $quantityiso,
        $operatingMode
    ) {
        if (!$this->validateValue($sender)
            || !$this->validateValue($depositorno)
            || !$this->validateValue($partnertype)
            || !$this->validateValue($partnerno)
            || !$this->validateValue($receiver)
            || !$this->validateValue($plant)
            || !$this->validateValue($quantityiso)
            || !$this->validateValue($operatingMode)
        ) {
            return false;
        }
        return true; //validation passed
    }

    /**
     * Check if a value is empty or not a generic name
     * @param mixed $value the configuration value
     * @return bool TRUE: value is correct || FALSE: value is not correct
     */
    private function validateValue($value)
    {
        if (empty($value) || !Validate::isGenericName($value)) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Create a new order request for YellowCube
     * @param mixed $id Order ID
     */
    public function reSendOrder($id)
    {
        try {
            $prestaGateway = new PrestaShopGateway();

            $orderRef = $prestaGateway->getOrderReferencefromYC($id);

            if (!$this->checkIfOrderExistInDBWithStatus10($orderRef)) {
                $order = PrestaShopGateway::getOrderByReference($orderRef);
                try {
                    $orderData = $this->createOrderObject(
                        $orderRef,
                        $order[0]['id_address_delivery'],
                        $order[0]['id_lang'],
                        $order[0]['id_carrier'],
                        $order[0]['date_upd'],
                        $order[0]['id_order']
                    );
                } catch (NoDeliveryException $noDevEx) {
                    $this->noDeliverySlipHandler($noDevEx->getMessage(), $noDevEx->orderRef);
                    return true;
                }

                WabService::retryInsert($orderData, $id);
            } else {
                Logger::addLog(
                    "Order ".$orderRef." has already Status 10. Please Check the YellowCube BackOffice Page",
                    4
                );
            }
        } catch (Exception $e) {
            $message = htmlentities($e->getMessage());
            Logger::addLog("Error reSending WAB to YellowCube. Message: ".$message." Orderreference: ".$orderRef, 4);
        }
    }

    /**
    * Sends a order to yellowcube when a specific state is set.
    * @param mixed $params PrestaShop Event parameters
    */
    public function hookactionOrderStatusUpdate($params)
    {
        try {
            $prestaGateway = new PrestaShopGateway();
            $order = $prestaGateway->getOrderByID($params['id_order']);
            $csvReader = new CsvReader();
            $paymentArray = $csvReader->readConfig(Constants::STATUSFILENAME);
            if (in_array($params['newOrderStatus']->id, $paymentArray) ||
                $params['newOrderStatus']->name == Constants::STATE) {
                if (!$this->checkIfOrderExistInDB($order[0]['reference'])) {
                    try {
                        $orderData = $this->createOrderObject(
                            $order[0]['reference'],
                            $order[0]['id_address_delivery'],
                            $order[0]['id_lang'],
                            $order[0]['id_carrier'],
                            $order[0]['date_upd'],
                            $order[0]['id_order']
                        );
                    } catch (NoDeliveryException $noDevEx) {
                        $this->noDeliverySlipHandler($noDevEx->getMessage(), $noDevEx->orderRef);
                        return true;
                    }

                    WabService::insert($orderData);
                } else {
                    Logger::addLog(
                        "Order ".$order[0]['reference']." is already in database. Check the YC Order page",
                        4
                    );
                }
            }
        } catch (Exception $e) {
            $message = htmlentities($e->getMessage());
            $text = "Error Sending WAB to YellowCube. Order was NOT send to YellowCube. Message: ";
            $moreText = " and ShippingStatus: ".$params['newOrderStatus']->name;
            Logger::addLog(
                $text.$message." Orderreference: ".$order[0]['reference'].$moreText,
                4
            );
        }
    }
    /**
     * Sends a new order to yellowcube
     * @param mixed $params PrestaShop Event parameters
     */
    public function hookdisplayOrderConfirmation($params)
    {
        if ((int)PrestaShopGateway::getConfig(Constants::WABCRON) == 0) {
            try {
                //When Lock exist exit function
                if (!Lock::checkExistingLocks($params['order']->reference)) {
                    $key = Lock::createLock($params['order']->reference);
                    try {
                        $csvReader = new CsvReader();
                        $paymentArray = $csvReader->readConfig(Constants::PAYMENTFILENAME);

                        if (in_array($params['order']->payment, $paymentArray)) {
                            return true;
                        }

                        if (!$this->checkIfOrderExistInDB($params['order']->reference)) {
                            try {
                                $orderData = $this->createOrderObject(
                                    $params['order']->reference,
                                    $params['order']->id_address_delivery,
                                    $params['order']->id_lang,
                                    $params['order']->id_carrier,
                                    $params['order']->date_upd,
                                    $params['order']->id
                                );
                            } catch (NoDeliveryException $noDevEx) {
                                $this->noDeliverySlipHandler($noDevEx->getMessage(), $noDevEx->orderRef);
                                return true;
                            }
                            WabService::insertWithKey($orderData, $key);
                        }
                    } catch (Exception $e) {
                        $message = htmlentities($e->getMessage());
                        $text = "Error Sending WAB to YellowCube. Order was NOT send to YellowCube. Message: ";
                        Logger::addLog(
                            $text.$message." Orderreference: ".$params['order']->reference,
                            4
                        );
                    }
                }
            } catch (PrestaShopDatabaseException $ex) {
                $message = htmlentities($e->getMessage());
                $text = "Double sending Orderreference: ";
                Logger::addLog(
                    $text.$params['order']->reference." lock already exist Message: ".$message,
                    4
                );
            } catch (Exception $aaex) {
                $message = htmlentities($e->getMessage());
                $text = "Unexpected Error while creating Lock for Order: ";
                Logger::addLog(
                    $text.$params['order']->reference." Message: ".$message,
                    4
                );
            }
        } else {
            Logger::addLog("WAB Cron activated", 1);
        }
    }

    /**
     * Checked if an order already was send to YellowCube
     * @param mixed $ref PrestaShop Order Reference
     * @return bool TRUE: Order exist || FALSE: Order doesn't exist
     */
    private function checkIfOrderExistInDB($ref)
    {
        $prestaGateway = new PrestaShopGateway();

        $array = $prestaGateway->getYCOrderbyReference($ref);

        if (count((array)$array) > 0) {
            return true;
        }

        return false;
    }

    /**
     * Checked if an order with state 10 in DB exist
     * @param mixed $ref PrestaShop Order Reference
     * @return bool TRUE: Order exist || FALSE: Order doesn't exist
     */
    private function checkIfOrderExistInDBWithStatus10($ref)
    {
        $prestaGateway = new PrestaShopGateway();

        $array = $prestaGateway->getYCOrderbyReference($ref);

        if ((int)$array[0]['statuscode'] == 10) {
            return true;
        }

        return false;
    }

    /**
     * Create an OrderObject with all the necessary Data for YellowCube
     * @param mixed $reference Order Reference<
     * @param mixed $id_address_delivery Address Delivery ID
     * @param mixed $id_lang Language ID
     * @param mixed $id_carrier Carrier ID
     * @param mixed $date_upd Date updated
     * @param mixed $order_id Order ID
     * @return OrderData an object with the required data for yellowcube transfer
     */
    private function createOrderObject($reference, $id_address_delivery, $id_lang, $id_carrier, $date_upd, $order_id)
    {
        $prestaGateway = new PrestaShopGateway();

        $orderData = new OrderData();

        $orderData->Sender = PrestaShopGateway::getConfig(Constants::SENDER);

        $orderData->DepositorNo = PrestaShopGateway::getConfig(Constants::DEPOSITOR_NO);

        $orderData->PartnerType = PrestaShopGateway::getConfig(Constants::PARTNER_TYPE);

        $orderData->PartnerNo = PrestaShopGateway::getConfig(Constants::PARTNER_NO);

        $orderData->CustomerOrderNo = $reference;

        $adressArray = $prestaGateway->getAdressArray($id_address_delivery);

        $orderData->Name1 = $adressArray['name1'];
        $orderData->Street = $adressArray['address1'];
        $orderData->ZIPCode = $adressArray['postcode'];
        $orderData->City = $adressArray['city'];
        $orderData->CountryCode = $adressArray['iso_code'];
        $orderData->PartnerReference = $adressArray['dni'];

        $additionalAddressArray = $this->createAdditionalAdress($adressArray);

        $orderData->Name2 = array_key_exists('Name2', $additionalAddressArray) ? $additionalAddressArray['Name2'] : '';
        $orderData->Name3 = array_key_exists('Name3', $additionalAddressArray) ? $additionalAddressArray['Name3'] : '';
        $orderData->Name4 = array_key_exists('Name4', $additionalAddressArray) ? $additionalAddressArray['Name4'] : '';

        $orderData->LanguageCode = $prestaGateway->getLangISO($id_lang);

        $carrier = $prestaGateway->getCarrierName($id_carrier);
        $carrierArray = $this->selectCarrier($carrier);

        $orderData->BasicShippingServices = $carrierArray[0];
        $orderData->AdditionalShippingService = $carrierArray[1];

        $dateTime = date_parse($date_upd);

        $day_new = ($dateTime['day'] < 10 ? '0'.$dateTime['day'] : $dateTime['day']);
        $month_new = ($dateTime['month'] < 10 ? '0'.$dateTime['month'] : $dateTime['month']);
        $orderData->CustomerOrderDate = $dateTime['year'].$month_new.$day_new;

        $orderData->OrderPosition = $prestaGateway->getOrdersPosition($reference);

        $orderData->OrderDocumentsFlag = (int)PrestaShopGateway::getConfig(Constants::DELIVERYATT);

        $orderData->DocStream = ($orderData->OrderDocumentsFlag == 1 ?
            $prestaGateway->generateDeliverySlipPDFByIdOrder($order_id) : '');

        return $orderData;
    }

    /**
     * Create the additional adress array for YellowCube namefields
     * @param array $adressArray The adress Array
     * @return mixed the additional Adress Array
     */
    private function createAdditionalAdress($adressArray)
    {
        $additionalAddressArray = array();

        if ($adressArray['company'] != null) {
            $this->addValueToNameAddressArray($additionalAddressArray, $adressArray['company']);
        }

        if ($adressArray['address2'] != null) {
            $this->addValueToNameAddressArray($additionalAddressArray, $adressArray['address2']);
        }

        if ($adressArray['other'] != null) {
            $textArray = $this->readAdresszeilenFromString($adressArray['other']);

            foreach ($textArray as $value) {
                $this->addValueToNameAddressArray($additionalAddressArray, $value);
            }
        }

        return $additionalAddressArray;
    }

    /**
     * Fills a value in the array till no namefield is available
     * @param mixed $additionalAddressArray the additional address Array
     * @param mixed $value the value to fill in the array
     */
    private function addValueToNameAddressArray(&$additionalAddressArray, $value) //& für Reference Übergabe
    {
        if (array_key_exists('Name2', $additionalAddressArray)) {
            if (array_key_exists('Name3', $additionalAddressArray)) {
                if (!array_key_exists('Name4', $additionalAddressArray)) {
                    $additionalAddressArray['Name4'] = $value;
                }
            } else {
                $additionalAddressArray['Name3'] = $value;
            }
        } else {
            $additionalAddressArray['Name2'] = $value;
        }
    }

    /**
     * Reads the [AL] lines from the other field
     * @param mixed $input Data from th Other addressfield
     * @return array An array with the extra addressfields
     */
    private function readAdresszeilenFromString($input)
    {
        $index = 0;
        $textArray = array();
        while (true) {
            $index =  strpos($input, Constants::AL, $index);
            if (is_bool($index)) {
                break;
            }

            $index = $index + 13;
            $tempStr = Tools::substr($input, $index);
            $tempInd = strpos($tempStr, Constants::AL);
            if (!is_bool($tempInd)) {
                array_push($textArray, Tools::substr($tempStr, 0, $tempInd));
            } else {
                array_push($textArray, Tools::substr($tempStr, 0));
            }
        }

        return $textArray;
    }

    /**
     * Reads the carrier
     * @param mixed $carrier carrier ID
     * @return array the carrier array
     */
    private function selectCarrier($carrier)
    {
        $csvReader = new CsvReader();

        $shippingArray = $csvReader->readCarriers(Constants::SHIPPINGFILENAME);

        $result = array();

        $result[0] = $carrier;
        $result[1] = "";

        if (count((array)$shippingArray) > 0) {
            //Additional carriers like PRI;SA
            foreach ($shippingArray as $data) {
                if ($carrier == $data[0]) {
                    $result[0] = $data[1];
                    $result[1] = $data[2];
                    break;
                }
            }
        }

        return $result;
    }

    /**
     * Run the WAB functionality
     */
    public function runWABJobs()
    {
        if ($this->active) {
            $this->updateNotFullProcessedYCOrders();
        }
    }

    /**
     * Updates the not full processed yellowcube orders
     */
    private function updateNotFullProcessedYCOrders()
    {
        try {
            //Update ART
            $bll = new BLL();
            $bll->updatePendingART();
            Lock::removeOldLocks();

            $prestaGateway = new PrestaShopGateway();

            $dataArr = $prestaGateway->getNotFullProcessedYCOrders();

            foreach ($dataArr as $data) {
                try {
                    WabService::update($data['id'], $data['yc_reference']);
                } catch (Exception $e) {
                    $message = $e->getMessage();
                    Logger::addLog(
                        "Error Updating WAB from YellowCube. YC_Ref: ".$data['yc_reference']." Message: ".$message,
                        4
                    );
                }
            }
        } catch (Exception $e) {
            $message = htmlentities($e->getMessage());
            Logger::addLog("Error Updating WAB from YellowCube. Message: ".$message, 4);
        }
    }


    //--WAR

    /**
     * Run the WAR functionality
     */
    public function runWARJobs()
    {
        if ($this->active) {
            $this->updateWAR();
        }
    }

    /**
     * Update the WAR for statuscode 100 yellowcube orders
     */
    private function updateWAR()
    {
        try {
            $prestaGateway = new PrestaShopGateway();
            $orderReferences = $prestaGateway->getWAROrders();

            foreach ($orderReferences as $ref) {
                try {
                    WabService::updateWar($ref['presta_reference']);
                } catch (Exception $e) {
                    $message = htmlentities($e->getMessage());
                    $text = "Error Updating WAR for Order: ";
                    Logger::addLog(
                        $text.$ref['presta_reference']." from YellowCube. Message: ".$message,
                        4
                    );
                }
            }
        } catch (Exception $e) {
            $message = htmlentities($e->getMessage());
            Logger::addLog("Error Updating WAR from YellowCube. Message: ".$message, 4);
        }
    }

    //--WAB Sending

    /**
     * Run the WARB send functionality
     */
public function runWABSendJobs(): void // No return value
{
    if ($this->active) {
        $this->sendWAB();
    }
}
    /**
     * Send new Orders with specified OrderStatus to YellowCube
     */
    private function sendWAB()
    {
        try {
            if ((int)PrestaShopGateway::getConfig(Constants::WABCRON) == 1) {
                Logger::addLog("WAB_SEND Cronjob started", 1);

                $prestaGateway = new PrestaShopGateway();
                $csvReader = new CsvReader();

                $statusArray = $csvReader->readConfig(Constants::CRONWABFILENAME);

                foreach ($statusArray as $status) {
                    $orders = $prestaGateway->getOrdersByStatus($status);

                    foreach ($orders as $orderarr) {
                        $order = (object)$orderarr;//cast single arrayobject to object

                        if (!$this->checkIfOrderExistInDB($order->reference)) {
                            $orderData = $this->createOrderObject(
                                $order->reference,
                                $order->id_address_delivery,
                                $order->id_lang,
                                $order->id_carrier,
                                $order->date_upd,
                                $order->id_order
                            );

                            WabService::insert($orderData);
                        }
                    }
                }
                Logger::addLog("WAB_SEND Cronjob finished", 1);
            }
        } catch (Exception $e) {
            $message = htmlentities($e->getMessage());
            Logger::addLog(
                "CronJob: Error Sending WAB Order: ".$order->reference." to YellowCube. Message: ".$message,
                4
            );
        }
    }

    /**
     * Run the BAR functionality
     */
public function runBar(): void // Explicit return type (no return value)
{
    if ($this->active) {
        $this->updateBar();
    }
}


    /**
     * Updates the YellowCube Inventory
     * Update: Erweitere updateBar
     */
private function updateBar(): void // Explicit return type (no return value)
{
    $bll = new BLL();
    $bll->processingBAR();
}



    /**
     * Refreshs the BAR Data for the YellowCube Inventory Tab
     * Update: Erweitere refresh, falls kein Eintrag erfolgt
	* Deaktiviere Log-Eintrag beim Start
     */
public function refreshtBarData(): void
{
    try {
/**        Logger::addLog("Start Refresh YellowCube Bar Data", 1, null, null, null, true); */
        $bll = new BLL();
        $bll->removeUnusedProductsFromBar();
        $bll->checkForNewProducts();
        $bll->updatePendingART();
        Logger::addLog("Update YellowCube BAR", 1, null, null, null, true);
    } catch (Exception $e) {
        $message = htmlspecialchars($e->getMessage(), ENT_QUOTES); // Use htmlspecialchars for safer encoding
        Logger::addLog("Error: refreshtBarData $e / Error Updating ART Data. Message: " . $message, 4, null, null, null, true);
    }
}


    /**
     * Syncs selected product with yellowcube
     * @param mixed $product_reference product reference
     * @return bool answer
     */
    public function syncART($product_reference)
    {
        $bll = new BLL();
        return $bll->syncArt($product_reference);
    }

    /**
     * Deactivates selected product from yellowcube
     * @param mixed $product_reference product reference
     * @return bool answer
     */
    public function deactivateART($product_reference)
    {
        $bll = new BLL();
        return $bll->deactivateART($product_reference);
    }


    /**
     * Create the Orderreference array from bar objects
     * @param Bar[] $bars bar object
     * @return array reference array
     */
    private function createReferenceArray($bars)
    {
        $references = array();

        foreach ($bars as $value) {
            $references[] = $value->reference;
        }

        return $references;
    }

    /**
     * Handels the NoDeliverySlip Exception
     * @param mixed $message the Exception message
     * @param mixed $orderRef the PrestaShop order reference
     */
    private function noDeliverySlipHandler($message, $orderRef)
    {
        Logger::addLog($message, 4);
        $prestaGateway = new PrestaShopGateway();
        $prestaGateway->errorInsertYellowCubeOrder($message, Constants::STATUSNOK, $orderRef);
    }
}
